using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Net;
using System.Text.RegularExpressions;
using System.Drawing;
using System.Diagnostics;

namespace Departure
{
    class BuildDataThread
    {
        ContainerControl m_sender = null;

        Delegate m_senderDelegate = null;

        int m_StationIndex;
        string m_Source = "";

        frmSettings m_Settings;

        public BuildDataThread(ContainerControl sender, Delegate senderDelegate, int StationIndex, string Source, frmSettings Settings)
		{
			m_sender = sender;
			m_senderDelegate = senderDelegate;
            m_StationIndex = StationIndex;
            m_Source = Source;
            m_Settings = Settings;

		}

		/// <summary>
		/// Method for ThreadStart delegate
		/// </summary>
		public void RunProcess()
		{
			Thread.CurrentThread.IsBackground = true; //make them a daemon
			LocalRunProcess();
		}

		/// <summary>
		/// Local Method for the actual work.
		/// </summary>
        private void LocalRunProcess()
        {
            if (m_Settings.useWAP)
            {
                GetTrainsWAP();
            }
            else
            {
                GetTrainsHTML();
            }
        }

        private void GetTrainsWAP()
        {
            try
            {
                ListView m_lv = new ListView();
                m_lv.Columns.Add("Time", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Train", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Direction", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Platform", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Later", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("FullDateTime", 0, HorizontalAlignment.Left);

                string Time;
                string Train;
                string Direction;
                string Platform;
                string Later;
                string FullDateTime;

                int currentItem = 0;
                int allItems = 0;

                if (m_Source.IndexOf("realtime information") >= 0 || m_Source.IndexOf("Real-time information") >= 0)
                {

                    Regex regex = new Regex("---<br/>.*?<br/>.*?<br/>", RegexOptions.Singleline | RegexOptions.IgnoreCase);

                    MatchCollection matchcollection = regex.Matches(m_Source);

                    allItems = matchcollection.Count;

                    foreach (Match tmpmatch in matchcollection)
                    {
                        currentItem++;

                        m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "", false, false, true, currentItem, allItems });

                        regex = new Regex("<{0,1}.b>", RegexOptions.Singleline | RegexOptions.IgnoreCase);

                        string[] matchcollection2 = regex.Split(tmpmatch.Value);

                        if (matchcollection2.Length == 5)
                        {

                            Time = ClearTime(ClearText(Regex.Replace(matchcollection2[3], "<.*?>", ""))).Trim();
                            Train = ClearTrain(ClearText(Regex.Replace(matchcollection2[1], "<.*?>", ""))).Trim();
                            Direction = ClearDirection(ClearText(Regex.Replace(matchcollection2[2], "<.*?>", ""))).Trim();
                            Platform = ClearText(Regex.Replace(ClearPlatform(GetPlatformWAP(matchcollection2[4])), "<.*?>", "")).Trim();
                            Later = ClearLater(ClearText(Regex.Replace(GetLaterWAP(matchcollection2[4]), "<.*?>", ""))).Trim();

                            DateTime FullDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, int.Parse(Time.Split(':')[0]), int.Parse(Time.Split(':')[1]), 0);

                            if (FullDate < DateTime.Now)
                                FullDate = FullDate.AddDays(1);

                            FullDateTime = FullDate.ToFileTimeUtc().ToString();

                            if (FilterData(m_StationIndex, Train, Direction, Platform))
                            {
                                m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, Time, Train, Direction, Platform, Later, FullDateTime, "", false, false, false, 0, 0});
                            }

                        }
                    }

                }
                else
                {
                    m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "No departures found", true, false, false, 0, 0 });
                }

                m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "", false, true, false, 0, 0 });
            }
            catch (Exception ex)
            {
                m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", ex.Message, true, false, 0, 0 });
            }
        }

        private void GetTrainsHTML()
        {
            try
            {
                ListView m_lv = new ListView();
                m_lv.Columns.Add("Time", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Train", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Direction", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Platform", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("Later", 60, HorizontalAlignment.Left);
                m_lv.Columns.Add("FullDateTime", 0, HorizontalAlignment.Left);

                string Time;
                string Train;
                string Direction;
                string Platform;
                string Later;
                string FullDateTime;

                int currentItem = 0;
                int allItems = 0;

                if (m_Source.IndexOf("Departures valid on") >= 0)
                {

                    m_Source = m_Source.Substring(m_Source.IndexOf("Departures valid on"), m_Source.Length - m_Source.IndexOf("Departures valid on") - 1);

                    Regex regex = new Regex("<table.*?>.*?</table>", RegexOptions.Singleline | RegexOptions.IgnoreCase);
                    Match match;
                    MatchCollection matchcollection;
                    MatchCollection matchcollection2;

                    matchcollection = regex.Matches(m_Source);

                    if (matchcollection.Count >= 5)
                    {

                        match = matchcollection[4];

                        regex = new Regex("<tr.*?</tr>", RegexOptions.Singleline | RegexOptions.IgnoreCase);

                        matchcollection = regex.Matches(match.Value);

                        allItems = matchcollection.Count;

                        foreach (Match tmpmatch in matchcollection)
                        {
                            currentItem++;

                            m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "", false, false, true, currentItem, allItems });

                            regex = new Regex("<td.*?</td>", RegexOptions.Singleline | RegexOptions.IgnoreCase);

                            matchcollection2 = regex.Matches(tmpmatch.Value);

                            if (matchcollection2.Count == 5 || matchcollection2.Count == 7 || matchcollection2.Count == 9)
                            {

                                Time = ClearTime(ClearText(Regex.Replace(matchcollection2[0].Value, "<.*?>", ""))).Trim();
                                Train = ClearTrain(ClearText(Regex.Replace(matchcollection2[2].Value, "<.*?>", ""))).Trim();
                                Direction = ClearText(Regex.Replace(matchcollection2[4].Value, "<.*?>", "")).Trim();
                                if (matchcollection2.Count == 7 || matchcollection2.Count == 9)
                                    Platform = ClearText(Regex.Replace(ClearPlatform(matchcollection2[6].Value), "<.*?>", "")).Trim();
                                else
                                    Platform = "";
                                if (matchcollection2.Count == 9)
                                    Later = ClearLater(ClearText(Regex.Replace(matchcollection2[8].Value, "<.*?>", ""))).Trim();
                                else
                                    Later = "";

                                DateTime FullDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, int.Parse(Time.Split(':')[0]), int.Parse(Time.Split(':')[1]), 0);

                                if (FullDate < DateTime.Now.AddMinutes(-2))
                                    FullDate = FullDate.AddDays(1);

                                FullDateTime = FullDate.ToFileTimeUtc().ToString();

                                if (FilterData(m_StationIndex, Train, Direction, Platform))
                                {
                                    m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, Time, Train, Direction, Platform, Later, FullDateTime, "", false, false, false, 0, 0 });
                                }

                            }
                        }

                    }
                    else
                    {
                        m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "No departures found", true, false,0,0 });
                    }

                }
                else
                {
                    m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "No departures found", true, false, false, 0, 0 });
                }

                m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", "", false, true, false, 0, 0 });
            }
            catch (Exception ex)
            {
                m_sender.BeginInvoke(m_senderDelegate, new object[] { m_StationIndex, "", "", "", "", "", "", ex.Message, true, false, 0, 0 });
            }
        }


        private bool FilterData(int index, string Train, string Direction, string Platform)
        {
            Regex regex;

            bool filterhitTrain = false;
            bool filterhitDirection = false;
            bool filterhitPlatform = false;
            
            switch (index)
            {
                case 0:
                    // Bus Filter
                    regex = new Regex("^Bus", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked1[7].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // IRE Filter
                    regex = new Regex("^IRE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked1[3].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RE Filter
                    regex = new Regex("^RE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked1[4].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RB Filter
                    regex = new Regex("^RB", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked1[5].ToString() == "0" && regex.Match(Train).Success)
                        return false;

                    // Train Filter
                    if (m_Settings.useTrainsFilter1)
                    {
                        regex = new Regex(m_Settings.TrainsFilter1.Replace(";","|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Train).Success)
                        {
                            filterhitTrain = true;
                        }
                    }

                    // Direction Filter
                    if (m_Settings.useDirectionsFilter1)
                    {
                        regex = new Regex(m_Settings.DirectionsFilter1.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Direction).Success)
                        {
                            filterhitDirection = true;
                        }
                    }

                    // Platform Filter
                    if (m_Settings.usePlatformsFilter1)
                    {
                        regex = new Regex(m_Settings.PlatformsFilter1.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Platform).Success)
                        {
                            filterhitPlatform = true;
                        }
                    }

                    if (m_Settings.useAndFilter1)
                    {
                        if (m_Settings.useTrainsFilter1 == true && m_Settings.useDirectionsFilter1 == false && m_Settings.usePlatformsFilter1 == false)
                            return filterhitTrain;
                        else if (m_Settings.useTrainsFilter1 == true && m_Settings.useDirectionsFilter1 == true && m_Settings.usePlatformsFilter1 == false)
                            return (filterhitTrain && filterhitDirection);
                        else if (m_Settings.useTrainsFilter1 == true && m_Settings.useDirectionsFilter1 == true && m_Settings.usePlatformsFilter1 == true)
                            return (filterhitTrain && filterhitDirection && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter1 == true && m_Settings.useDirectionsFilter1 == false && m_Settings.usePlatformsFilter1 == true)
                            return (filterhitTrain && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter1 == false && m_Settings.useDirectionsFilter1 == false && m_Settings.usePlatformsFilter1 == true)
                            return (filterhitPlatform);
                        else //if (m_Settings.useTrainsFilter1 == false && m_Settings.useDirectionsFilter1 == true && m_Settings.usePlatformsFilter1 == false)
                            return (filterhitDirection);
                    }
                    else
                        if (m_Settings.useTrainsFilter1 == true || m_Settings.useDirectionsFilter1 == true || m_Settings.usePlatformsFilter1 == true)
                            return (filterhitTrain || filterhitDirection || filterhitPlatform);
                        else
                            return true;

                case 1:
                    // Bus Filter
                    regex = new Regex("^Bus", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked2[7].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // IRE Filter
                    regex = new Regex("^IRE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked2[3].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RE Filter
                    regex = new Regex("^RE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked2[4].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RB Filter
                    regex = new Regex("^RB", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked2[5].ToString() == "0" && regex.Match(Train).Success)
                        return false;

                    // Train Filter
                    if (m_Settings.useTrainsFilter2)
                    {
                        regex = new Regex(m_Settings.TrainsFilter2.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Train).Success)
                        {
                            filterhitTrain = true;
                        }
                    }

                    // Direction Filter
                    if (m_Settings.useDirectionsFilter2)
                    {
                        regex = new Regex(m_Settings.DirectionsFilter2.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Direction).Success)
                        {
                            filterhitDirection = true;
                        }
                    }

                    // Platform Filter
                    if (m_Settings.usePlatformsFilter2)
                    {
                        regex = new Regex(m_Settings.PlatformsFilter2.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Platform).Success)
                        {
                            filterhitPlatform = true;
                        }
                    }

                    if (m_Settings.useAndFilter2)
                    {
                        if (m_Settings.useTrainsFilter2 == true && m_Settings.useDirectionsFilter2 == false && m_Settings.usePlatformsFilter2 == false)
                            return filterhitTrain;
                        else if (m_Settings.useTrainsFilter2 == true && m_Settings.useDirectionsFilter2 == true && m_Settings.usePlatformsFilter2 == false)
                            return (filterhitTrain && filterhitDirection);
                        else if (m_Settings.useTrainsFilter2 == true && m_Settings.useDirectionsFilter2 == true && m_Settings.usePlatformsFilter2 == true)
                            return (filterhitTrain && filterhitDirection && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter2 == true && m_Settings.useDirectionsFilter2 == false && m_Settings.usePlatformsFilter2 == true)
                            return (filterhitTrain && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter2 == false && m_Settings.useDirectionsFilter2 == false && m_Settings.usePlatformsFilter2 == true)
                            return (filterhitPlatform);
                        else //if (m_Settings.useTrainsFilter2 == false && m_Settings.useDirectionsFilter2 == true && m_Settings.usePlatformsFilter2 == false)
                            return (filterhitDirection);
                    }
                    else
                        if (m_Settings.useTrainsFilter2 == true || m_Settings.useDirectionsFilter2 == true || m_Settings.usePlatformsFilter2 == true)
                            return (filterhitTrain || filterhitDirection || filterhitPlatform);
                        else
                            return true;

                case 2:
                    // Bus Filter
                    regex = new Regex("^Bus", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked3[7].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // IRE Filter
                    regex = new Regex("^IRE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked3[3].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RE Filter
                    regex = new Regex("^RE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked3[4].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RB Filter
                    regex = new Regex("^RB", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked3[5].ToString() == "0" && regex.Match(Train).Success)
                        return false;

                    // Train Filter
                    if (m_Settings.useTrainsFilter3)
                    {
                        regex = new Regex(m_Settings.TrainsFilter3.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Train).Success)
                        {
                            filterhitTrain = true;
                        }
                    }

                    // Direction Filter
                    if (m_Settings.useDirectionsFilter3)
                    {
                        regex = new Regex(m_Settings.DirectionsFilter3.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Direction).Success)
                        {
                            filterhitDirection = true;
                        }
                    }

                    // Platform Filter
                    if (m_Settings.usePlatformsFilter3)
                    {
                        regex = new Regex(m_Settings.PlatformsFilter3.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Platform).Success)
                        {
                            filterhitPlatform = true;
                        }
                    }

                    if (m_Settings.useAndFilter3)
                    {
                        if (m_Settings.useTrainsFilter3 == true && m_Settings.useDirectionsFilter3 == false && m_Settings.usePlatformsFilter3 == false)
                            return filterhitTrain;
                        else if (m_Settings.useTrainsFilter3 == true && m_Settings.useDirectionsFilter3 == true && m_Settings.usePlatformsFilter3 == false)
                            return (filterhitTrain && filterhitDirection);
                        else if (m_Settings.useTrainsFilter3 == true && m_Settings.useDirectionsFilter3 == true && m_Settings.usePlatformsFilter3 == true)
                            return (filterhitTrain && filterhitDirection && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter3 == true && m_Settings.useDirectionsFilter3 == false && m_Settings.usePlatformsFilter3 == true)
                            return (filterhitTrain && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter3 == false && m_Settings.useDirectionsFilter3 == false && m_Settings.usePlatformsFilter3 == true)
                            return (filterhitPlatform);
                        else //if (m_Settings.useTrainsFilter3 == false && m_Settings.useDirectionsFilter3 == true && m_Settings.usePlatformsFilter3 == false)
                            return (filterhitDirection);
                    }
                    else
                        if (m_Settings.useTrainsFilter3 == true || m_Settings.useDirectionsFilter3 == true || m_Settings.usePlatformsFilter3 == true)
                            return (filterhitTrain || filterhitDirection || filterhitPlatform);
                        else
                            return true;

                case 3:
                    // Bus Filter
                    regex = new Regex("^Bus", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked4[7].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // IRE Filter
                    regex = new Regex("^IRE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked4[3].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RE Filter
                    regex = new Regex("^RE", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked4[4].ToString() == "0" && regex.Match(Train).Success)
                        return false;
                    // RB Filter
                    regex = new Regex("^RB", RegexOptions.IgnoreCase);
                    if (m_Settings.TransfersChecked4[5].ToString() == "0" && regex.Match(Train).Success)
                        return false;

                    // Train Filter
                    if (m_Settings.useTrainsFilter4)
                    {
                        regex = new Regex(m_Settings.TrainsFilter4.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Train).Success)
                        {
                            filterhitTrain = true;
                        }
                    }

                    // Direction Filter
                    if (m_Settings.useDirectionsFilter4)
                    {
                        regex = new Regex(m_Settings.DirectionsFilter4.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Direction).Success)
                        {
                            filterhitDirection = true;
                        }
                    }

                    // Platform Filter
                    if (m_Settings.usePlatformsFilter4)
                    {
                        regex = new Regex(m_Settings.PlatformsFilter4.Replace(";", "|"), RegexOptions.IgnoreCase);
                        if (regex.Match(Platform).Success)
                        {
                            filterhitPlatform = true;
                        }
                    }

                    if (m_Settings.useAndFilter4)
                    {
                        if (m_Settings.useTrainsFilter4 == true && m_Settings.useDirectionsFilter4 == false && m_Settings.usePlatformsFilter4 == false)
                            return filterhitTrain;
                        else if (m_Settings.useTrainsFilter4 == true && m_Settings.useDirectionsFilter4 == true && m_Settings.usePlatformsFilter4 == false)
                            return (filterhitTrain && filterhitDirection);
                        else if (m_Settings.useTrainsFilter4 == true && m_Settings.useDirectionsFilter4 == true && m_Settings.usePlatformsFilter4 == true)
                            return (filterhitTrain && filterhitDirection && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter4 == true && m_Settings.useDirectionsFilter4 == false && m_Settings.usePlatformsFilter4 == true)
                            return (filterhitTrain && filterhitPlatform);
                        else if (m_Settings.useTrainsFilter4 == false && m_Settings.useDirectionsFilter4 == false && m_Settings.usePlatformsFilter4 == true)
                            return (filterhitPlatform);
                        else //if (m_Settings.useTrainsFilter4 == false && m_Settings.useDirectionsFilter4 == true && m_Settings.usePlatformsFilter4 == false)
                            return (filterhitDirection);
                    }
                    else
                        if (m_Settings.useTrainsFilter4 == true || m_Settings.useDirectionsFilter4 == true || m_Settings.usePlatformsFilter4 == true)
                            return (filterhitTrain || filterhitDirection || filterhitPlatform);
                        else
                            return true;

            }

            return true;
        }

        private string ClearText(string input)
        {
            input = input.Replace("\n", "");
            input = input.Replace("\r", "");

            input = input.Replace("&gt;", ">");
            input = input.Replace("&lt;", "<");

            input = input.Replace("&nbsp;", " ");		//no-breakspace=non-breakingspace
            input = input.Replace("&iexcl;", "");		//invertedexclamationmark
            input = input.Replace("&cent;", "");		//centsign
            input = input.Replace("&pound;", "");		//poundsign
            input = input.Replace("&curren;", "");		//currencysign
            input = input.Replace("&yen;", "");		//yensign=yuansign
            input = input.Replace("&brvbar;", "");		//brokenbar=brokenverticalbar
            input = input.Replace("&sect;", "");		//sectionsign
            input = input.Replace("&uml;", "");		//diaeresis=spacingdiaeresis(Umlaut)
            input = input.Replace("&copy;", "");		//copyrightsign
            input = input.Replace("&ordf;", "");		//feminineordinalindicator
            input = input.Replace("&laquo;", "");		//left-pointingdoubleanglequotationmark
            input = input.Replace("&not;", "");		//notsign
            input = input.Replace("&shy;", "");		//softhyphen=discretionaryhyphen
            input = input.Replace("&reg;", "");		//registeredsign=registeredtrademarksig
            input = input.Replace("&macr;", "");		//macron=spacingmacron=overline=APLov
            input = input.Replace("&deg;", "");		//degreesign
            input = input.Replace("&plusmn;", "");		//plus-minussign=plus-or-minussign
            input = input.Replace("&sup2;", "");		//superscripttwo=superscriptdigittwo
            input = input.Replace("&sup3;", "");		//superscriptthree=superscriptdigitthree
            input = input.Replace("&acute;", "");		//acuteaccent=spacingacute
            input = input.Replace("&micro;", "");		//microsign
            input = input.Replace("&para;", "");		//pilcrowsign=paragraphsign
            input = input.Replace("&middot;", "");		//middledot=Georgiancomma=Greekmiddle
            input = input.Replace("&cedil;", "");		//cedilla=spacingcedilla
            input = input.Replace("&sup1;", "");		//superscriptone=superscriptdigitone
            input = input.Replace("&ordm;", "");		//masculineordinalindicator
            input = input.Replace("&raquo;", "");		//right-pointingdoubleanglequotationmark
            input = input.Replace("&frac14;", "");		//vulgarfractiononequarter
            input = input.Replace("&frac12;", "");		//vulgarfractiononehalf
            input = input.Replace("&frac34;", "");		//vulgarfractionthreequarters
            input = input.Replace("&iquest;", "");		//invertedquestionmark
            input = input.Replace("&Agrave;", "");		//latincapitalletterAwithgrave
            input = input.Replace("&Aacute;", "");		//latincapitalletterAwithacute
            input = input.Replace("&Acirc;", "");		//latincapitalletterAwithcircumflex
            input = input.Replace("&Atilde;", "");		//latincapitalletterAwithtilde
            input = input.Replace("&Auml;", "");		//latincapitalletterAwithdiaeresis
            input = input.Replace("&Aring;", "");		//latincapitalletterAwithringabove
            input = input.Replace("&AElig;", "");		//latincapitalletterAE
            input = input.Replace("&Ccedil;", "");		//latincapitalletterCwithcedilla
            input = input.Replace("&Egrave;", "");		//latincapitalletterEwithgrave
            input = input.Replace("&Eacute;", "");		//latincapitalletterEwithacute
            input = input.Replace("&Ecirc;", "");		//latincapitalletterEwithcircumflex
            input = input.Replace("&Euml;", "");		//latincapitalletterEwithdiaeresis
            input = input.Replace("&Igrave;", "");		//latincapitalletterIwithgrave
            input = input.Replace("&Iacute;", "");		//latincapitalletterIwithacute
            input = input.Replace("&Icirc;", "");		//latincapitalletterIwithcircumflex
            input = input.Replace("&Iuml;", "");		//latincapitalletterIwithdiaeresis
            input = input.Replace("&ETH;", "");		//latincapitalletterETH
            input = input.Replace("&Ntilde;", "");		//latincapitalletterNwithtilde
            input = input.Replace("&Ograve;", "");		//latincapitalletterOwithgrave
            input = input.Replace("&Oacute;", "");		//latincapitalletterOwithacute
            input = input.Replace("&Ocirc;", "");		//latincapitalletterOwithcircumflex
            input = input.Replace("&Otilde;", "");		//latincapitalletterOwithtilde
            input = input.Replace("&Ouml;", "");		//latincapitalletterOwithdiaeresis
            input = input.Replace("&times;", "");		//multiplicationsign
            input = input.Replace("&Oslash;", "");		//latincapitalletterOwithstroke
            input = input.Replace("&Ugrave;", "");		//latincapitalletterUwithgrave
            input = input.Replace("&Uacute;", "");		//latincapitalletterUwithacute
            input = input.Replace("&Ucirc;", "");		//latincapitalletterUwithcircumflex
            input = input.Replace("&Uuml;", "");		//latincapitalletterUwithdiaeresis
            input = input.Replace("&Yacute;", "");		//latincapitalletterYwithacute
            input = input.Replace("&THORN;", "");		//latincapitalletterTHORN
            input = input.Replace("&szlig;", "");		//latinsmalllettersharps=ess-zed
            input = input.Replace("&agrave;", "");		//latinsmallletterawithgrave
            input = input.Replace("&aacute;", "");		//latinsmallletterawithacute
            input = input.Replace("&acirc;", "");		//latinsmallletterawithcircumflex
            input = input.Replace("&atilde;", "");		//latinsmallletterawithtilde
            input = input.Replace("&auml;", "");		//latinsmallletterawithdiaeresis
            input = input.Replace("&aring;", "");		//latinsmallletterawithringabove
            input = input.Replace("&aelig;", "");		//latinsmallletterae
            input = input.Replace("&ccedil;", "");		//latinsmalllettercwithcedilla
            input = input.Replace("&egrave;", "");		//latinsmallletterewithgrave
            input = input.Replace("&eacute;", "");		//latinsmallletterewithacute
            input = input.Replace("&ecirc;", "");		//latinsmallletterewithcircumflex
            input = input.Replace("&euml;", "");		//latinsmallletterewithdiaeresis
            input = input.Replace("&igrave;", "");		//latinsmallletteriwithgrave
            input = input.Replace("&iacute;", "");		//latinsmallletteriwithacute
            input = input.Replace("&icirc;", "");		//latinsmallletteriwithcircumflex
            input = input.Replace("&iuml;", "");		//latinsmallletteriwithdiaeresis
            input = input.Replace("&eth;", "");		//latinsmalllettereth
            input = input.Replace("&ntilde;", "");		//latinsmallletternwithtilde
            input = input.Replace("&ograve;", "");		//latinsmallletterowithgrave
            input = input.Replace("&oacute;", "");		//latinsmallletterowithacute
            input = input.Replace("&ocirc;", "");		//latinsmallletterowithcircumflex
            input = input.Replace("&otilde;", "");		//latinsmallletterowithtilde
            input = input.Replace("&ouml;", "");		//latinsmallletterowithdiaeresis
            input = input.Replace("&divide;", "");		//divisionsign
            input = input.Replace("&oslash;", "");		//latinsmallletterowithstroke
            input = input.Replace("&ugrave;", "");		//latinsmallletteruwithgrave
            input = input.Replace("&uacute;", "");		//latinsmallletteruwithacute
            input = input.Replace("&ucirc;", "");		//latinsmallletteruwithcircumflex
            input = input.Replace("&uuml;", "");		//latinsmallletteruwithdiaeresis
            input = input.Replace("&yacute;", "");		//latinsmallletterywithacute
            input = input.Replace("&thorn;", "");		//latinsmallletterthorn
            input = input.Replace("&yuml;", "");		//latinsmallletterywithdiaeresis
            input = input.Replace("&#160;", " ");		//no-breakspace=non-breakingspace
            input = input.Replace("&#161;", "");		//invertedexclamationmark
            input = input.Replace("&#162;", "");		//centsign
            input = input.Replace("&#163;", "");		//poundsign
            input = input.Replace("&#164;", "");		//currencysign
            input = input.Replace("&#165;", "");		//yensign=yuansign
            input = input.Replace("&#166;", "");		//brokenbar=brokenverticalbar
            input = input.Replace("&#167;", "");		//sectionsign
            input = input.Replace("&#168;", "");		//diaeresis=spacingdiaeresis(Umlaut)
            input = input.Replace("&#169;", "");		//copyrightsign
            input = input.Replace("&#170;", "");		//feminineordinalindicator
            input = input.Replace("&#171;", "");		//left-pointingdoubleanglequotationmark
            input = input.Replace("&#172;", "");		//notsign
            input = input.Replace("&#173;", "");		//softhyphen=discretionaryhyphen
            input = input.Replace("&#174;", "");		//registeredsign=registeredtrademarksig
            input = input.Replace("&#175;", "");		//macron=spacingmacron=overline=APLov
            input = input.Replace("&#176;", "");		//degreesign
            input = input.Replace("&#177;", "");		//plus-minussign=plus-or-minussign
            input = input.Replace("&#178;", "");		//superscripttwo=superscriptdigittwo
            input = input.Replace("&#179;", "");		//superscriptthree=superscriptdigitthree
            input = input.Replace("&#180;", "");		//acuteaccent=spacingacute
            input = input.Replace("&#181;", "");		//microsign
            input = input.Replace("&#182;", "");		//pilcrowsign=paragraphsign
            input = input.Replace("&#183;", "");		//middledot=Georgiancomma=Greekmiddle
            input = input.Replace("&#184;", "");		//cedilla=spacingcedilla
            input = input.Replace("&#185;", "");		//superscriptone=superscriptdigitone
            input = input.Replace("&#186;", "");		//masculineordinalindicator
            input = input.Replace("&#187;", "");		//right-pointingdoubleanglequotationmark
            input = input.Replace("&#188;", "");		//vulgarfractiononequarter
            input = input.Replace("&#189;", "");		//vulgarfractiononehalf
            input = input.Replace("&#190;", "");		//vulgarfractionthreequarters
            input = input.Replace("&#191;", "");		//invertedquestionmark
            input = input.Replace("&#192;", "");		//latincapitalletterAwithgrave
            input = input.Replace("&#193;", "");		//latincapitalletterAwithacute
            input = input.Replace("&#194;", "");		//latincapitalletterAwithcircumflex
            input = input.Replace("&#195;", "");		//latincapitalletterAwithtilde
            input = input.Replace("&#196;", "");		//latincapitalletterAwithdiaeresis
            input = input.Replace("&#197;", "");		//latincapitalletterAwithringabove
            input = input.Replace("&#198;", "");		//latincapitalletterAE
            input = input.Replace("&#199;", "");		//latincapitalletterCwithcedilla
            input = input.Replace("&#200;", "");		//latincapitalletterEwithgrave
            input = input.Replace("&#201;", "");		//latincapitalletterEwithacute
            input = input.Replace("&#202;", "");		//latincapitalletterEwithcircumflex
            input = input.Replace("&#203;", "");		//latincapitalletterEwithdiaeresis
            input = input.Replace("&#204;", "");		//latincapitalletterIwithgrave
            input = input.Replace("&#205;", "");		//latincapitalletterIwithacute
            input = input.Replace("&#206;", "");		//latincapitalletterIwithcircumflex
            input = input.Replace("&#207;", "");		//latincapitalletterIwithdiaeresis
            input = input.Replace("&#208;", "");		//latincapitalletterETH
            input = input.Replace("&#209;", "");		//latincapitalletterNwithtilde
            input = input.Replace("&#210;", "");		//latincapitalletterOwithgrave
            input = input.Replace("&#211;", "");		//latincapitalletterOwithacute
            input = input.Replace("&#212;", "");		//latincapitalletterOwithcircumflex
            input = input.Replace("&#213;", "");		//latincapitalletterOwithtilde
            input = input.Replace("&#214;", "");		//latincapitalletterOwithdiaeresis
            input = input.Replace("&#215;", "");		//multiplicationsign
            input = input.Replace("&#216;", "");		//latincapitalletterOwithstroke
            input = input.Replace("&#217;", "");		//latincapitalletterUwithgrave
            input = input.Replace("&#218;", "");		//latincapitalletterUwithacute
            input = input.Replace("&#219;", "");		//latincapitalletterUwithcircumflex
            input = input.Replace("&#220;", "");		//latincapitalletterUwithdiaeresis
            input = input.Replace("&#221;", "");		//latincapitalletterYwithacute
            input = input.Replace("&#222;", "");		//latincapitalletterTHORN
            input = input.Replace("&#223;", "");		//latinsmalllettersharps=ess-zed
            input = input.Replace("&#224;", "");		//latinsmallletterawithgrave
            input = input.Replace("&#225;", "");		//latinsmallletterawithacute
            input = input.Replace("&#226;", "");		//latinsmallletterawithcircumflex
            input = input.Replace("&#227;", "");		//latinsmallletterawithtilde
            input = input.Replace("&#228;", "");		//latinsmallletterawithdiaeresis
            input = input.Replace("&#229;", "");		//latinsmallletterawithringabove
            input = input.Replace("&#230;", "");		//latinsmallletterae
            input = input.Replace("&#231;", "");		//latinsmalllettercwithcedilla
            input = input.Replace("&#232;", "");		//latinsmallletterewithgrave
            input = input.Replace("&#233;", "");		//latinsmallletterewithacute
            input = input.Replace("&#234;", "");		//latinsmallletterewithcircumflex
            input = input.Replace("&#235;", "");		//latinsmallletterewithdiaeresis
            input = input.Replace("&#236;", "");		//latinsmallletteriwithgrave
            input = input.Replace("&#237;", "");		//latinsmallletteriwithacute
            input = input.Replace("&#238;", "");		//latinsmallletteriwithcircumflex
            input = input.Replace("&#239;", "");		//latinsmallletteriwithdiaeresis
            input = input.Replace("&#240;", "");		//latinsmalllettereth
            input = input.Replace("&#241;", "");		//latinsmallletternwithtilde
            input = input.Replace("&#242;", "");		//latinsmallletterowithgrave
            input = input.Replace("&#243;", "");		//latinsmallletterowithacute
            input = input.Replace("&#244;", "");		//latinsmallletterowithcircumflex
            input = input.Replace("&#245;", "");		//latinsmallletterowithtilde
            input = input.Replace("&#246;", "");		//latinsmallletterowithdiaeresis
            input = input.Replace("&#247;", "");		//divisionsign
            input = input.Replace("&#248;", "");		//latinsmallletterowithstroke
            input = input.Replace("&#249;", "");		//latinsmallletteruwithgrave
            input = input.Replace("&#250;", "");		//latinsmallletteruwithacute
            input = input.Replace("&#251;", "");		//latinsmallletteruwithcircumflex
            input = input.Replace("&#252;", "");		//latinsmallletteruwithdiaeresis
            input = input.Replace("&#253;", "");		//latinsmallletterywithacute
            input = input.Replace("&#254;", "");		//latinsmallletterthorn
            input = input.Replace("&#255;", "");		//latinsmallletterywithd


            return input;
        }

        private string ClearLater(string input)
        {
            input = input.Replace("on time", "");
            input = input.Replace("-", "");
            input = input.Replace("n.a.", "");


            return input;
        }

        private string ClearPlatform(string input)
        {
            input = Regex.Replace(input, "<br>.*", "", RegexOptions.IgnoreCase | RegexOptions.Singleline);

            input = input.Replace("Platf.", "");

            return input;
        }

        private string ClearTime(string input)
        {
            input = input.Replace(" ", "");

            return input;
        }

        private string ClearDirection(string input)
        {
            input = input.Replace("<", "");
            input = input.Replace(">", "");

            return input;
        }

        private string ClearTrain(string input)
        {
            input = input.Replace("S     ", "");
            input = input.Replace("S    ", "");

            input = input.Replace("U     ", "");
            input = input.Replace("U    ", "");

            input = input.Replace(" ", "");

            return input;
        }

        private string GetPlatformWAP(string input)
        {
            string[] output = Regex.Split(input,"&nbsp;&nbsp;");

            if (output.Length == 3)
                return output[2];
            else
                return "";
        }

        private string GetLaterWAP(string input)
        {
            string[] output = Regex.Split(input, "&nbsp;&nbsp;");
            if (output.Length > 1)
                return output[1];
            else
                return "";
        }

    }
}
