using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;
using System.Threading;
using System.Net;

using DirkRettschlag.Utils;
using System.Diagnostics;

namespace Departure
{
    public partial class frmMain : Form
    {
        frmSettings settings = new frmSettings();

        bool Station1Updating;
        bool Station2Updating;
        bool Station3Updating;
        bool Station4Updating;

        delegate void RequestDelegate(int StationIndex, string Source, bool Error);
        delegate void BuildDataDelegate(int StationIndex, string Time, string Train, string Direction, string Platform, string Later, string FullDateTime, string Error, bool isError, bool done, bool update, int currentitem, int allitems);

        public frmMain()
        {
           
            Debug.WriteLine("Started: " + DateTime.Now.ToString());

            Debug.WriteLine("Initializing: " + DateTime.Now.ToString());

            // http://msdn2.microsoft.com/en-us/library/aa446535.aspx

            InitializeComponent();

            this.tbcMain.Top = 0;
            this.tbcMain.Left = 0;
            this.tbcMain.Height = this.ClientSize.Height;
            this.tbcMain.Width = this.ClientSize.Width;

            Debug.WriteLine("Initialized: " + DateTime.Now.ToString());

        }

        private void mniExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void mniRefresh_Click(object sender, EventArgs e)
        {
            if (mniRefresh.Text == "Refresh")
            {
                try
                {
                    RefreshData(tbcMain.SelectedIndex);
                }
                catch (Exception ex)
                {
                    Cursor.Current = Cursors.Default;

                    MessageBox.Show(ex.Message, "An error occured");
                }
            }
            else if (mniRefresh.Text == "Stop")
            {
                ctrlCountdown.Stop();
                mniRefresh.Text = "";
                mniRefresh.Enabled = false;
            }
        }

        private void RefreshData(int index)
        {

            if (Connected() || settings.useAutoConnect)
            {
                //MessageBox.Show("Connected");
                Cursor.Current = Cursors.WaitCursor;
                mniRefresh.Enabled = false;

                switch (index)
                {
                    case 0:
                        labelProgressStation1.Text = "Retrieving Data..."; 
                        labelProgressStation1.Visible = true;
                        lock (lsvMainStation1)
                            lsvMainStation1.Items.Clear();
                        Station1Updating = true;
                        break;
                    case 1:
                        labelProgressStation2.Text = "Retrieving Data...";
                        labelProgressStation2.Visible = true;
                        lock (lsvMainStation2)
                            lsvMainStation2.Items.Clear();
                        Station2Updating = true;
                        break;
                    case 2:
                        labelProgressStation3.Text = "Retrieving Data...";
                        labelProgressStation3.Visible = true;
                        lock (lsvMainStation3)
                            lsvMainStation3.Items.Clear();
                        Station3Updating = true;
                        break;
                    case 3:
                        labelProgressStation4.Text = "Retrieving Data...";
                        labelProgressStation4.Visible = true;
                        lock (lsvMainStation4)
                            lsvMainStation4.Items.Clear();
                        Station4Updating = true;
                        break;

                }

                GetData(index);
            }
           
        }

        private string AppPath()
        {
            return System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
        }

        private void ReadData()
        {
            ListViewItem lvi;
            string[] itemArray;

            if (File.Exists(AppPath() + "\\Station1.csv"))
            {
                StreamReader sr = new StreamReader(AppPath() + "\\Station1.csv", Encoding.Default);

                while (sr.Peek() >= 0)
                {
                    itemArray = sr.ReadLine().Split(new Char[] { '' });

                    lvi = new ListViewItem(itemArray[0]);
                    lvi.SubItems.Add(itemArray[1]);
                    lvi.SubItems.Add(itemArray[2]);
                    lvi.SubItems.Add(itemArray[3]);
                    lvi.SubItems.Add(itemArray[4]);
                    if (itemArray[4] != "")
                    {
                        lvi.ForeColor = Color.Red;
                    }
                    lvi.SubItems.Add(itemArray[5]);
                    lock (lsvMainStation1)
                        Invoking.AddItem(lsvMainStation1, lvi);
                }
            }

            if (File.Exists(AppPath() + "\\Station2.csv"))
            {
                StreamReader sr = new StreamReader(AppPath() + "\\Station2.csv", Encoding.Default);

                while (sr.Peek() >= 0)
                {
                    itemArray = sr.ReadLine().Split(new Char[] { '' });

                    lvi = new ListViewItem(itemArray[0]);
                    lvi.SubItems.Add(itemArray[1]);
                    lvi.SubItems.Add(itemArray[2]);
                    lvi.SubItems.Add(itemArray[3]);
                    lvi.SubItems.Add(itemArray[4]);
                    if (itemArray[4] != "")
                    {
                        lvi.ForeColor = Color.Red;
                    }
                    lvi.SubItems.Add(itemArray[5]);
                    lock (lsvMainStation2)
                        Invoking.AddItem(lsvMainStation2, lvi);
                }
            }

            if (File.Exists(AppPath() + "\\Station3.csv"))
            {
                StreamReader sr = new StreamReader(AppPath() + "\\Station3.csv", Encoding.Default);

                while (sr.Peek() >= 0)
                {
                    itemArray = sr.ReadLine().Split(new Char[] { '' });

                    lvi = new ListViewItem(itemArray[0]);
                    lvi.SubItems.Add(itemArray[1]);
                    lvi.SubItems.Add(itemArray[2]);
                    lvi.SubItems.Add(itemArray[3]);
                    lvi.SubItems.Add(itemArray[4]);
                    if (itemArray[4] != "")
                    {
                        lvi.ForeColor = Color.Red;
                    }
                    lvi.SubItems.Add(itemArray[5]);
                    lock (lsvMainStation3)
                        Invoking.AddItem(lsvMainStation3, lvi);
                }
            }

            if (File.Exists(AppPath() + "\\Station4.csv"))
            {
                StreamReader sr = new StreamReader(AppPath() + "\\Station4.csv", Encoding.Default);

                while (sr.Peek() >= 0)
                {
                    itemArray = sr.ReadLine().Split(new Char[] { '' });

                    lvi = new ListViewItem(itemArray[0]);
                    lvi.SubItems.Add(itemArray[1]);
                    lvi.SubItems.Add(itemArray[2]);
                    lvi.SubItems.Add(itemArray[3]);
                    lvi.SubItems.Add(itemArray[4]);
                    if (itemArray[4] != "")
                    {
                        lvi.ForeColor = Color.Red;
                    }
                    lvi.SubItems.Add(itemArray[5]);
                    lock (lsvMainStation4)
                        Invoking.AddItem(lsvMainStation4, lvi);
                }
            }

        }

        private void SaveData(int index)
        {

            StreamWriter sw = new StreamWriter(AppPath() + "\\Station" + (index+1).ToString() + ".csv", false, Encoding.Default);

            switch (index)
            {
                case 0:
                    foreach (ListViewItem lvi in lsvMainStation1.Items)
                    {
                        sw.WriteLine(lvi.SubItems[0].Text + "" + lvi.SubItems[1].Text + "" + lvi.SubItems[2].Text + "" + lvi.SubItems[3].Text + "" + lvi.SubItems[4].Text + "" + lvi.SubItems[5].Text);
                    }
                    break;
                case 1:
                    foreach (ListViewItem lvi in lsvMainStation2.Items)
                    {
                        sw.WriteLine(lvi.SubItems[0].Text + "" + lvi.SubItems[1].Text + "" + lvi.SubItems[2].Text + "" + lvi.SubItems[3].Text + "" + lvi.SubItems[4].Text + "" + lvi.SubItems[5].Text);
                    }
                    break;
                case 2:
                    foreach (ListViewItem lvi in lsvMainStation3.Items)
                    {
                        sw.WriteLine(lvi.SubItems[0].Text + "" + lvi.SubItems[1].Text + "" + lvi.SubItems[2].Text + "" + lvi.SubItems[3].Text + "" + lvi.SubItems[4].Text + "" + lvi.SubItems[5].Text);
                    }
                    break;
                case 3:
                    foreach (ListViewItem lvi in lsvMainStation4.Items)
                    {
                        sw.WriteLine(lvi.SubItems[0].Text + "" + lvi.SubItems[1].Text + "" + lvi.SubItems[2].Text + "" + lvi.SubItems[3].Text + "" + lvi.SubItems[4].Text + "" + lvi.SubItems[5].Text);
                    }
                    break;

            }

            sw.Flush();
            sw.Close();
        }

        

        private void BuildData(string source, int index)
        {

            switch (index)
            {
                case 0:
                    labelProgressStation1.Text = "Filtering Data...";
                    lsvMainStation1.BeginUpdate();
                    break;
                case 1:
                    labelProgressStation2.Text = "Filtering Data...";
                    lsvMainStation2.BeginUpdate();
                    break;
                case 2:
                    labelProgressStation3.Text = "Filtering Data...";
                    lsvMainStation3.BeginUpdate();
                    break;
                case 3:
                    labelProgressStation4.Text = "Filtering Data...";
                    lsvMainStation4.BeginUpdate();
                    break;
            }

            BuildDataDelegate buildDataDone = new BuildDataDelegate(BuildData);

            BuildDataThread wt = new BuildDataThread(this, buildDataDone, index, source, settings);
            Thread t = new Thread(new ThreadStart(wt.RunProcess));
            t.IsBackground = true; //make them a daemon - prevent thread callback issues
            t.Start();
           
        }

        private void BuildData(int StationIndex, string Time, string Train, string Direction, string Platform, string Later, string FullDateTime, string Error, bool isError, bool done, bool update, int currentitem, int allitems)
        {
            if (isError)
            {
                switch (StationIndex)
                {
                    case 0:
                        labelProgressStation1.Text = "";
                        labelProgressStation1.Visible = false;
                        Station1Updating = false;
                        lsvMainStation1.EndUpdate();
                        break;
                    case 1:
                        labelProgressStation2.Text = "";
                        labelProgressStation2.Visible = false;
                        Station2Updating = false;
                        lsvMainStation2.EndUpdate();
                        break;
                    case 2:
                        labelProgressStation3.Text = "";
                        labelProgressStation3.Visible = false;
                        Station3Updating = false;
                        lsvMainStation3.EndUpdate();
                        break;
                    case 3:
                        labelProgressStation4.Text = "";
                        labelProgressStation4.Visible = false;
                        Station4Updating = false;
                        lsvMainStation4.EndUpdate();
                        break;
                }

                Cursor.Current = Cursors.Default;
                MessageBox.Show(Error, "An error occured");
            }
            else if (done)
            {
                if (!backgroundWorkerLoadSave.IsBusy)
                    backgroundWorkerLoadSave.RunWorkerAsync(new object[] { "Save", StationIndex });

                switch (StationIndex)
                {
                    case 0:
                        labelProgressStation1.Text = "";
                        labelProgressStation1.Visible = false;
                        ResizeColumns(lsvMainStation1);
                        Station1Updating = false;
                        lsvMainStation1.EndUpdate();
                        break;
                    case 1:
                        labelProgressStation2.Text = "";
                        labelProgressStation2.Visible = false;
                        ResizeColumns(lsvMainStation2);
                        Station2Updating = false;
                        lsvMainStation2.EndUpdate();
                        break;
                    case 2:
                        labelProgressStation3.Text = "";
                        labelProgressStation3.Visible = false;
                        ResizeColumns(lsvMainStation3);
                        Station3Updating = false;
                        lsvMainStation3.EndUpdate();
                        break;
                    case 3:
                        labelProgressStation4.Text = "";
                        labelProgressStation4.Visible = false;
                        ResizeColumns(lsvMainStation4);
                        Station4Updating = false;
                        lsvMainStation4.EndUpdate();
                        break;
                }

                SetCursorButton();

            }
            else if (update)
            {
                switch (StationIndex)
                {
                    case 0:
                        labelProgressStation1.Text = string.Format("Filtering Trains ({0}/{1})", currentitem, allitems);
                        break;
                    case 1:
                        labelProgressStation2.Text = string.Format("Filtering Trains ({0}/{1})", currentitem, allitems);
                        break;
                    case 2:
                        labelProgressStation3.Text = string.Format("Filtering Trains ({0}/{1})", currentitem, allitems);
                        break;
                    case 3:
                        labelProgressStation4.Text = string.Format("Filtering Trains ({0}/{1})", currentitem, allitems);
                        break;
                }
            }
            else
            {
                ListViewItem lvi;

                switch (StationIndex)
                {
                    case 0:
                        lvi = new ListViewItem(Time);

                        lvi.SubItems.Add(Train);
                        lvi.SubItems.Add(Direction);
                        lvi.SubItems.Add(Platform);
                        lvi.SubItems.Add(Later);
                        lvi.SubItems.Add(FullDateTime);

                        if (Later != "")
                        {
                            lvi.ForeColor = Color.Red;
                        }

                        lock (lsvMainStation1)
                            lsvMainStation1.Items.Add(lvi);

                        break;
                    case 1:
                        lvi = new ListViewItem(Time);

                        lvi.SubItems.Add(Train);
                        lvi.SubItems.Add(Direction);
                        lvi.SubItems.Add(Platform);
                        lvi.SubItems.Add(Later);
                        lvi.SubItems.Add(FullDateTime);

                        if (Later != "")
                        {
                            lvi.ForeColor = Color.Red;
                        }


                        lock (lsvMainStation2)
                            lsvMainStation2.Items.Add(lvi);

                        break;
                    case 2:
                        lvi = new ListViewItem(Time);

                        lvi.SubItems.Add(Train);
                        lvi.SubItems.Add(Direction);
                        lvi.SubItems.Add(Platform);
                        lvi.SubItems.Add(Later);
                        lvi.SubItems.Add(FullDateTime);

                        if (Later != "")
                        {
                            lvi.ForeColor = Color.Red;
                        }

                        lock (lsvMainStation3)
                            lsvMainStation3.Items.Add(lvi);

                        break;
                    case 3:
                        lvi = new ListViewItem(Time);

                        lvi.SubItems.Add(Train);
                        lvi.SubItems.Add(Direction);
                        lvi.SubItems.Add(Platform);
                        lvi.SubItems.Add(Later);
                        lvi.SubItems.Add(FullDateTime);

                        if (Later != "")
                        {
                            lvi.ForeColor = Color.Red;
                        }

                        lock (lsvMainStation4)
                            lsvMainStation4.Items.Add(lvi);

                        break;

                }

            }
           
        }

        private void GetData(int index)
        {
            string URL = "";

            switch (index)
            {
                case 0:
                    if (settings.useWAP)
                    {
                        URL = "http://wap.bahn.de/bin/mobil/bhftafel.exe/eow?si=" + settings.Station1 + "&ti=actual&bt=dep&p=" + settings.Transfers1 + "&max=50&tn=&tns=1&start=yes";
                    }
                    else
                    {
                        URL = "http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + settings.Station1 + "&time=actual&boardType=dep&productsFilter=" + settings.Transfers1 + "&start=Anzeigen&distance=1";
                    }
                    break;
                case 1:
                    if (settings.useWAP)
                    {
                        URL = "http://wap.bahn.de/bin/mobil/bhftafel.exe/eow?si=" + settings.Station2 + "&ti=actual&bt=dep&p=" + settings.Transfers2 + "&max=50&tn=&tns=1&start=yes";
                    }
                    else
                    {
                        URL = "http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + settings.Station2 + "&time=actual&boardType=dep&productsFilter=" + settings.Transfers2 + "&start=Anzeigen&distance=1";
                    }
                    break;
                case 2:
                    if (settings.useWAP)
                    {
                        URL = "http://wap.bahn.de/bin/mobil/bhftafel.exe/eow?si=" + settings.Station3 + "&ti=actual&bt=dep&p=" + settings.Transfers3 + "&max=50&tn=&tns=1&start=yes";
                    }
                    else
                    {
                        URL = "http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + settings.Station3 + "&time=actual&boardType=dep&productsFilter=" + settings.Transfers3 + "&start=Anzeigen&distance=1";
                    }
                    break;
                case 3:
                    if (settings.useWAP)
                    {
                        URL = "http://wap.bahn.de/bin/mobil/bhftafel.exe/eow?si=" + settings.Station4 + "&ti=actual&bt=dep&p=" + settings.Transfers4 + "&max=50&tn=&tns=1&start=yes";
                    }
                    else
                    {
                        URL = "http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + settings.Station4 + "&time=actual&boardType=dep&productsFilter=" + settings.Transfers4 + "&start=Anzeigen&distance=1";
                    }
                    break;

            }

            RequestDelegate requestDone = new RequestDelegate(RequestDone);

            RequestThread wt = new RequestThread(this, requestDone, index, URL, settings.UserAgent, settings.HTTPHeader);
            Thread t = new Thread(new ThreadStart(wt.RunProcess));
            t.IsBackground = true; //make them a daemon - prevent thread callback issues
            t.Start();

        }

        private void RequestDone(int StationIndex, string Source, bool Error)
        {

            if (Error == false)
            {
                BuildData(Source, StationIndex);
                
            }
            else
            {
                switch (StationIndex)
                {
                    case 0:
                        labelProgressStation1.Text = "";
                        labelProgressStation1.Visible = false;
                        Station1Updating = false;
                        break;
                    case 1:
                        labelProgressStation2.Text = "";
                        labelProgressStation2.Visible = false;
                        Station2Updating = false;
                        break;
                    case 2:
                        labelProgressStation3.Text = "";
                        labelProgressStation3.Visible = false;
                        Station3Updating = false;
                        break;
                    case 3:
                        labelProgressStation4.Text = "";
                        labelProgressStation4.Visible = false;
                        Station4Updating = false;
                        break;
                }

                Cursor.Current = Cursors.Default;
                MessageBox.Show(Source, "An error occured");
            }
        }
                
        private void mniSettings_Click(object sender, EventArgs e)
        {
            settings.ShowDialog();

            if (settings.Abbreviation1 != "")
                tbpMainStation1.Text = settings.Abbreviation1;
            else
                tbpMainStation1.Text = "Station1";
            if (settings.Abbreviation2 != "")
                tbpMainStation2.Text = settings.Abbreviation2;
            else
                tbpMainStation2.Text = "Station2";
            if (settings.Abbreviation3 != "")
                tbpMainStation3.Text = settings.Abbreviation3;
            else
                tbpMainStation3.Text = "Station3";
            if (settings.Abbreviation4 != "")
                tbpMainStation4.Text = settings.Abbreviation4;
            else
                tbpMainStation4.Text = "Station4";

            tmrRefresh.Enabled = settings.useAutoUpdate;
           
            tmrRemove.Enabled = settings.useRemovePastDepartures;

        }

        private void ResizeColumns(ListView listview)
        {
            for (int i = 0; i < listview.Columns.Count - 1; i++)
            {
                listview.Columns[i].Width = -2;
            }

            listview.Columns[listview.Columns.Count - 1].Width = 0;
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            try
            {
                Debug.WriteLine("Loading: " + DateTime.Now.ToString());

                if (!backgroundWorkerLoadSave.IsBusy)
                    backgroundWorkerLoadSave.RunWorkerAsync(new object[] { "Load", -1 });

                //ReadData();

                ResizeColumns(lsvMainStation1);
                ResizeColumns(lsvMainStation2);
                ResizeColumns(lsvMainStation3);
                ResizeColumns(lsvMainStation4);

                if (settings.Abbreviation1 != "")
                    tbpMainStation1.Text = settings.Abbreviation1;
                else
                    tbpMainStation1.Text = "Station1";
                if (settings.Abbreviation2 != "")
                    tbpMainStation2.Text = settings.Abbreviation2;
                else
                    tbpMainStation2.Text = "Station2";
                if (settings.Abbreviation3 != "")
                    tbpMainStation3.Text = settings.Abbreviation3;
                else
                    tbpMainStation3.Text = "Station3";
                if (settings.Abbreviation4 != "")
                    tbpMainStation4.Text = settings.Abbreviation4;
                else
                    tbpMainStation4.Text = "Station4";

                tmrRefresh.Interval = settings.UpdateStations * 60 * 1000;

                if (settings.useAutoUpdate)
                {
                    tmrRefresh.Enabled = true;
                    tmrRefresh_Tick(null, null);
                }

                tmrRemove.Enabled = settings.useRemovePastDepartures;

                tbcMain.SelectedIndex = 0;

                Debug.WriteLine("Loaded: " + DateTime.Now.ToString());
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "An error occured");
            }

        }

        private void frmMain_Resize(object sender, EventArgs e)
        {
            ResizeColumns(lsvMainStation1);
            ResizeColumns(lsvMainStation2);
            ResizeColumns(lsvMainStation3);
            ResizeColumns(lsvMainStation4);

        }

        private void tbcMain_SelectedIndexChanged(object sender, EventArgs e)
        {
            SetCursorButton();            
        }

        private void SetCursorButton()
        {
            switch (tbcMain.SelectedIndex)
            {
                case 0:
                    mniRefresh.Text = "Refresh";
                    lsvMainStation1.Focus();
                    if (Station1Updating == true)
                    {
                        Cursor.Current = Cursors.WaitCursor;
                        mniRefresh.Enabled = false;
                    }
                    else
                    {
                        Cursor.Current = Cursors.Default;
                        mniRefresh.Enabled = true;
                    }
                    break;
                case 1:
                    lsvMainStation2.Focus();
                    mniRefresh.Text = "Refresh";
                    if (Station2Updating == true)
                    {
                        Cursor.Current = Cursors.WaitCursor;
                        mniRefresh.Enabled = false;
                    }
                    else
                    {
                        Cursor.Current = Cursors.Default;
                        mniRefresh.Enabled = true;
                    }
                    break;
                case 2:
                    lsvMainStation3.Focus();
                    mniRefresh.Text = "Refresh";
                    if (Station3Updating == true)
                    {
                        Cursor.Current = Cursors.WaitCursor;
                        mniRefresh.Enabled = false;
                    }
                    else
                    {
                        Cursor.Current = Cursors.Default;
                        mniRefresh.Enabled = true;
                    }
                    break;
                case 3:
                    lsvMainStation4.Focus();
                    mniRefresh.Text = "Refresh";
                    if (Station4Updating == true)
                    {
                        Cursor.Current = Cursors.WaitCursor;
                        mniRefresh.Enabled = false;
                    }
                    else
                    {
                        Cursor.Current = Cursors.Default;
                        mniRefresh.Enabled = true;
                    }
                    break;
                case 4:
                    Cursor.Current = Cursors.Default;
                    if (ctrlCountdown.isRunning)
                    {
                        mniRefresh.Enabled = true;
                        mniRefresh.Text = "Stop";
                    }
                    else
                    {
                        mniRefresh.Enabled = false;
                        mniRefresh.Text = "";
                    }
                    break;
                    
            }
        }

        private void tmrRefresh_Tick(object sender, EventArgs e)
        {
            if (settings.Station1 != "")
                RefreshData(0);
            if (settings.Station2 != "")
                RefreshData(1);
            if (settings.Station3 != "")
                RefreshData(2);
            if (settings.Station4 != "")
                RefreshData(3);
        }

        private bool Connected()
        {
            bool ret;
            try
            {
                string HostName = Dns.GetHostName();
                IPHostEntry thisHost = Dns.GetHostEntry(HostName);
                string thisIpAddr = thisHost.AddressList[0].ToString();
                ret = thisIpAddr != IPAddress.Parse("127.0.0.1").ToString();
            }
            catch
            {
                return false;
            }
            return ret;
        }

        private void mniCountdown_Click(object sender, EventArgs e)
        {
            string countdownTime = "";
            string train = "";
            string later = "";

            switch (tbcMain.SelectedIndex)
            {
                case 0:
                    if (lsvMainStation1.SelectedIndices.Count > 0)
                    {
                        countdownTime = lsvMainStation1.Items[lsvMainStation1.SelectedIndices[0]].SubItems[5].Text;
                        train = settings.CountdownTrainFormat.Replace("%TI%", lsvMainStation1.Items[lsvMainStation1.SelectedIndices[0]].SubItems[0].Text);
                        train = train.Replace("%TR%", lsvMainStation1.Items[lsvMainStation1.SelectedIndices[0]].SubItems[1].Text);
                        train = train.Replace("%DI%", lsvMainStation1.Items[lsvMainStation1.SelectedIndices[0]].SubItems[2].Text);
                        train = train.Replace("%PL%", lsvMainStation1.Items[lsvMainStation1.SelectedIndices[0]].SubItems[3].Text);
                        later = lsvMainStation1.Items[lsvMainStation1.SelectedIndices[0]].SubItems[4].Text;
                    }
                    break;
                case 1:
                    if (lsvMainStation2.SelectedIndices.Count > 0)
                    {
                        countdownTime = lsvMainStation2.Items[lsvMainStation2.SelectedIndices[0]].SubItems[5].Text;
                        train = settings.CountdownTrainFormat.Replace("%TI%", lsvMainStation2.Items[lsvMainStation2.SelectedIndices[0]].SubItems[0].Text);
                        train = train.Replace("%TR%", lsvMainStation2.Items[lsvMainStation2.SelectedIndices[0]].SubItems[1].Text);
                        train = train.Replace("%DI%", lsvMainStation2.Items[lsvMainStation2.SelectedIndices[0]].SubItems[2].Text);
                        train = train.Replace("%PL%", lsvMainStation2.Items[lsvMainStation2.SelectedIndices[0]].SubItems[3].Text);
                        later = lsvMainStation2.Items[lsvMainStation2.SelectedIndices[0]].SubItems[4].Text;
                    }
                    break;
                case 2:
                    if (lsvMainStation3.SelectedIndices.Count > 0)
                    {
                        countdownTime = lsvMainStation3.Items[lsvMainStation3.SelectedIndices[0]].SubItems[5].Text;
                        train = settings.CountdownTrainFormat.Replace("%TI%", lsvMainStation3.Items[lsvMainStation3.SelectedIndices[0]].SubItems[0].Text);
                        train = train.Replace("%TR%", lsvMainStation3.Items[lsvMainStation3.SelectedIndices[0]].SubItems[1].Text);
                        train = train.Replace("%DI%", lsvMainStation3.Items[lsvMainStation3.SelectedIndices[0]].SubItems[2].Text);
                        train = train.Replace("%PL%", lsvMainStation3.Items[lsvMainStation3.SelectedIndices[0]].SubItems[3].Text);
                        later = lsvMainStation3.Items[lsvMainStation3.SelectedIndices[0]].SubItems[4].Text;
                    }
                    break;
                case 3:
                    if (lsvMainStation4.SelectedIndices.Count > 0)
                    {
                        countdownTime = lsvMainStation4.Items[lsvMainStation4.SelectedIndices[0]].SubItems[5].Text;
                        train = settings.CountdownTrainFormat.Replace("%TI%", lsvMainStation4.Items[lsvMainStation4.SelectedIndices[0]].SubItems[0].Text);
                        train = train.Replace("%TR%", lsvMainStation4.Items[lsvMainStation4.SelectedIndices[0]].SubItems[1].Text);
                        train = train.Replace("%DI%", lsvMainStation4.Items[lsvMainStation4.SelectedIndices[0]].SubItems[2].Text);
                        train = train.Replace("%PL%", lsvMainStation4.Items[lsvMainStation4.SelectedIndices[0]].SubItems[3].Text);
                        later = lsvMainStation4.Items[lsvMainStation4.SelectedIndices[0]].SubItems[4].Text;
                    }
                    break;
            }

            if (countdownTime != "")
            {
                double correction = 0;
                if (later != "" && later.IndexOf("approx. ") >= 0)
                    correction = double.Parse(later.Replace("approx. ", "").Replace(" minutes later", ""));

                DateTime alarmTime = DateTime.FromFileTimeUtc(long.Parse(countdownTime)).AddMinutes(correction);

                ctrlCountdown.Start(alarmTime, train, later);
                tbcMain.SelectedIndex = 4;
            }
        }

        private void tmrRemove_Tick(object sender, EventArgs e)
        {
            if (!backgroundWorkerRemove.IsBusy && !backgroundWorkerLoadSave.IsBusy)
                backgroundWorkerRemove.RunWorkerAsync();
        }

        private void backgroundWorkerRemove_DoWork(object sender, DoWorkEventArgs e)
        {
            if (Station1Updating == false)
                RemovePastDepartures(1);
            if (Station2Updating == false)
                RemovePastDepartures(2);
            if (Station3Updating == false)
                RemovePastDepartures(3);
            if (Station4Updating == false)
                RemovePastDepartures(4);
        }

        private void RemovePastDepartures(int StationIndex)
        {
            ListView listview = new ListView();

            switch (StationIndex)
            {
                case 1:
                    listview = lsvMainStation1;
                    break;
                case 2:
                    listview = lsvMainStation2;
                    break;
                case 3:
                    listview = lsvMainStation3;
                    break;
                case 4:
                    listview = lsvMainStation4;
                    break;
            }

            try
            { 
                // Hack for ListViews without Items
                if (listview.Items.Count > 0)
                {
                }
            }
            catch
            {
                return;
            }
            
            if (listview.Items.Count > 0)
            {
                for (int i = listview.Items.Count - 1; i >= 0; i--)
                {
                    double correction = 0;
                    if (listview.Items[i].SubItems[4].Text != "" && listview.Items[i].SubItems[4].Text.IndexOf("approx. ") >= 0)
                        correction = double.Parse(listview.Items[i].SubItems[4].Text.Replace("approx. ", "").Replace(" minutes later", ""));

                    DateTime corrected = DateTime.FromFileTimeUtc(long.Parse(listview.Items[i].SubItems[5].Text)).AddMinutes(correction);
                    if (corrected < DateTime.Now)
                    {
                        lock (listview)
                            Invoking.RemoveItem(listview, i);
                    }
                }
            }

        }

        private void backgroundWorkerRemove_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private void backgroundWorkerLoadSave_DoWork(object sender, DoWorkEventArgs e)
        {
            if ((string)((Object[])e.Argument)[0] == "Load")
            {
                ReadData();
            }
            else if ((string)((Object[])e.Argument)[0] == "Save")
            {
                SaveData((int)((Object[])e.Argument)[1]);
            }
        }

        private void backgroundWorkerLoadSave_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
                throw e.Error;
        }

        private void frmMain_Closing(object sender, CancelEventArgs e)
        {
            
        }

        private void ctrlCountdown_OnCountdownFinished()
        {
            if (tbcMain.SelectedIndex == 4)
            {
               mniRefresh.Enabled = false;
               mniRefresh.Text = "";
            }
                   
        }


    }

    
}
