using System;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Collections.Generic;
using System.Net;
using System.IO;
using System.Text.RegularExpressions;
using DirkRettschlag.Utils;
using System.Data;

[WebService(Namespace = "http://rettschlag.com/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class DepartureWebService : System.Web.Services.WebService
{
    #region Variables

    HttpWebRequest request;
    List<string> deviceIDs;

    #endregion Variables

    #region Structs

    public struct GetDeparturesInput
    {
        public string Station;
        public string StationID;

        public string Transfers;

        public string TransfersSelected;

        public bool UseTrainsFilter;
        public string TrainsFilter;

        public bool UseDirectionsFilter;
        public string DirectionsFilter;

        public bool UsePlatformsFilter;
        public string PlatformsFilter;

        public bool UseAndFilter;
    }

    #endregion Structs

    #region Constructor

    public DepartureWebService()
    {
        //Uncomment the following line if using designed components 
        //InitializeComponent(); 

        deviceIDs = new List<string>();

        deviceIDs.Add("3FBF5000-7351-0801-1706-8AB2475903EB");
        deviceIDs.Add("00197735-0302-0115-0028-0050BF3F5173");
    }

    #endregion Constructor

    #region WebMethods

    [WebMethod(Description = "Get filtered departures", EnableSession = false)]
    public string GetDepartures(string DeviceID, GetDeparturesInput input)
    {
        return getDepartures(DeviceID, input);
    }

    [WebMethod(Description = "Find stations matching the given input.", EnableSession = false)]
    public List<string> FindStation(string DeviceID, string Name)
    {
        return findStation(DeviceID, Name);
    }

    #endregion WebMethods

    #region Private Methods

    private List<string> findStation(string DeviceID, string Name)
    {
        if (!deviceIDs.Contains(DeviceID))
        {
            throw new Exception("You are not allowed to use this Webservice");
        }

        List<string> stations = new List<string>();

        request = (HttpWebRequest)WebRequest.Create("http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + Name + "&time=actual&boardType=dep&productsFilter=0000001000000000&start=Anzeigen&distance=1");

        request.Credentials = CredentialCache.DefaultCredentials;
        request.KeepAlive = true;
        request.Method = "GET";

        HttpWebResponse response = (HttpWebResponse)request.GetResponse();

        StreamReader sr = new StreamReader(response.GetResponseStream());

        string source = sr.ReadToEnd();

        sr.Close();

        if (source.IndexOf("Your input is ambiguous.") >= 0)
        {
            Regex regex = new Regex("<select name=\"input\".*?</select>", RegexOptions.Singleline | RegexOptions.IgnoreCase);
            Match match;
            MatchCollection matchcollection;

            match = regex.Match(source);

            if (match.Success)
            {

                regex = new Regex("<option.*?>.*?</option>", RegexOptions.IgnoreCase);

                matchcollection = regex.Matches(match.Value);

                string station;

                foreach (Match tmpmatch in matchcollection)
                {
                    station = Regex.Replace(Utilities.ClearText(tmpmatch.Value), "<option value=\"", "");
                    station = Regex.Replace(station, "\".*", "");
                    stations.Add(station);
                }

            }
        }
        else
        {

            stations.Add(Name);
        }

        return stations;
    }

    private string getDepartures(string DeviceID, GetDeparturesInput input)
    {
        if (!deviceIDs.Contains(DeviceID))
        {
            throw new Exception("You are not allowed to use this Webservice");
        }

        string source = getSource(input);

        return getDepartureXML(source, input);

    }

    private string getSource(GetDeparturesInput input)
    {
        string URL = "";
        if (input.StationID == "")
            URL = "http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + input.Station + "&time=actual&boardType=dep&productsFilter=" + input.Transfers + "&start=Anzeigen&distance=1";
        else
            URL = "http://reiseauskunft.bahn.de/bin/bhftafel.exe/en?L=ua_pda&input=" + input.StationID + "&time=actual&boardType=dep&productsFilter=" + input.Transfers + "&start=Anzeigen&distance=1";

        request = (HttpWebRequest)WebRequest.Create(URL);

        request.Credentials = CredentialCache.DefaultCredentials;
        request.KeepAlive = true;
        request.Method = "GET";

        HttpWebResponse response = (HttpWebResponse)request.GetResponse();

        System.IO.StreamReader sr = new System.IO.StreamReader(response.GetResponseStream());

        string source = sr.ReadToEnd();

        sr.Close();
        return source;
    }

    private string getDepartureXML(string source, GetDeparturesInput input)
    {
        DataTable dt = new DataTable("Departure");

        dt.Columns.Add("Time", typeof(string));
        dt.Columns.Add("Train", typeof(string));
        dt.Columns.Add("Direction", typeof(string));
        dt.Columns.Add("Platform", typeof(string));
        dt.Columns.Add("Later", typeof(string));
        dt.Columns.Add("Full_DateTime", typeof(string));
    
        try
        {

            string Time;
            string Train;
            string Direction;
            string Platform;
            string Later;
            string FullDateTime;

            int currentItem = 0;
            int allItems = 0;

            if (source.IndexOf("Departures valid on") >= 0)
            {
                Regex regexCountColumns = new Regex("<th class=\"resultdark.*?>.+?</th>", RegexOptions.IgnoreCase | RegexOptions.Multiline | RegexOptions.CultureInvariant | RegexOptions.Compiled);

                int countColumns = regexCountColumns.Matches(source).Count;

                Regex regex;
                if (countColumns == 3)
                {
                    regex = new Regex("<tr.*?>.*?<td.*?>.*?<span.*?>(?<Time>.*?)</span>.*?</td>.*?<td.*?>.*?<a href.*?>(?<Train>.*?)</a>.*?</td>.*?<td.*?>.*?<a href.*?>(?<Direction>.*?)</a>.*?</td>.*?</tr>", RegexOptions.Singleline | RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
                }
                else if (countColumns == 4)
                {
                    regex = new Regex("<tr.*?>.*?<td.*?>.*?<span.*?>(?<Time>.*?)</span>.*?</td>.*?<td.*?>.*?<a href.*?>(?<Train>.*?)</a>.*?</td>.*?<td.*?>.*?<a href.*?>(?<Direction>.*?)</a>.*?</td>.*?<td.*?></td>.*?<td.*?>(?<Platform>.*?)</td>.*?</tr>", RegexOptions.Singleline | RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
                }
                else if (countColumns == 5)
                {
                    regex = new Regex("<tr.*?>.*?<td.*?>.*?<span.*?>(?<Time>.*?)</span>.*?</td>.*?<td.*?>.*?<a href.*?>(?<Train>.*?)</a>.*?</td>.*?<td.*?>.*?<a href.*?>(?<Direction>.*?)</a>.*?</td>.*?<td.*?></td>.*?<td.*?>(?<Platform>.*?)</td>.*?<td.*?></td>.*?<td.*?>.*?<span.*?><span.*?>(?<Later>.*?)</span>.*?</td>.*?</tr>", RegexOptions.Singleline | RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
                }
                else
                {
                    throw new Exception("Source not valid.");
                }

                source = Regex.Replace(source, "^.*?Time&nbsp;</th>", "", RegexOptions.Singleline | RegexOptions.CultureInvariant | RegexOptions.Compiled);
                source = Regex.Replace(source, "</table>.*", "", RegexOptions.Singleline | RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);

                MatchCollection matchcollection = regex.Matches(source);

                allItems = Regex.Matches(source, "<tr.*?>.*?</tr>", RegexOptions.Singleline | RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled).Count;

                if (allItems > 0)
                {

                    foreach (Match tmpmatch in matchcollection)
                    {
                        currentItem++;

                        Time = ClearTime(Utilities.ClearText(tmpmatch.Groups["Time"].Value)).Trim();
                        Train = ClearTrain(Utilities.ClearText(tmpmatch.Groups["Train"].Value)).Trim();
                        Direction = Utilities.ClearText(tmpmatch.Groups["Direction"].Value).Trim();

                        if (tmpmatch.Groups.Count >= 5)
                            Platform = ClearPlatform(Utilities.ClearText(tmpmatch.Groups["Platform"].Value)).Trim();
                        else
                            Platform = "";
                        if (tmpmatch.Groups.Count >= 6)
                            Later = ClearLater(Utilities.ClearText(tmpmatch.Groups["Later"].Value)).Trim();
                        else
                            Later = "";

                        if (FilterData(Train, Direction, Platform, input))
                        {
                            DateTime FullDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, int.Parse(Time.Split(':')[0]), int.Parse(Time.Split(':')[1]), 0);

                            if (FullDate < DateTime.Now.AddMinutes(-2))
                                FullDate = FullDate.AddDays(1);

                            FullDateTime = FullDate.ToFileTimeUtc().ToString();

                            DataRow dr = dt.NewRow();
                            dr["Time"] = Time;
                            dr["Train"] = Train;
                            dr["Direction"] = Direction;
                            dr["Platform"] = Platform;
                            dr["Later"] = Later;
                            dr["Full_DateTime"] = FullDateTime;

                            dt.Rows.Add(dr);
                        }
                    }
                }
            }
        }
        catch (Exception ex)
        {
            throw ex;
        }

        StringWriter xmlOut = new StringWriter();
        dt.WriteXml(xmlOut);
        return xmlOut.ToString();

    }

    private bool FilterData(string Train, string Direction, string Platform, GetDeparturesInput input)
    {
        Regex regex;

        bool filterhitTrain = false;
        bool filterhitDirection = false;
        bool filterhitPlatform = false;

        // Bus Filter
        regex = new Regex("^Bus", RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
        if (input.TransfersSelected[7].ToString() == "0" && regex.Match(Train).Success)
            return false;
        // IRE Filter
        regex = new Regex("^IRE", RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
        if (input.TransfersSelected[3].ToString() == "0" && regex.Match(Train).Success)
            return false;
        // RE Filter
        regex = new Regex("^RE", RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
        if (input.TransfersSelected[4].ToString() == "0" && regex.Match(Train).Success)
            return false;
        // RB Filter
        regex = new Regex("^RB", RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
        if (input.TransfersSelected[5].ToString() == "0" && regex.Match(Train).Success)
            return false;

        // Train Filter
        if (input.UseTrainsFilter)
        {
            regex = new Regex(input.TrainsFilter.Replace(";", "|"), RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
            if (regex.Match(Train).Success)
            {
                filterhitTrain = true;
            }
        }

        // Direction Filter
        if (input.UseDirectionsFilter)
        {
            regex = new Regex(input.DirectionsFilter.Replace(";", "|"), RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
            if (regex.Match(Direction).Success)
            {
                filterhitDirection = true;
            }
        }

        // Platform Filter
        if (input.UsePlatformsFilter)
        {
            regex = new Regex(input.PlatformsFilter.Replace(";", "|"), RegexOptions.IgnoreCase | RegexOptions.CultureInvariant | RegexOptions.Compiled);
            if (regex.Match(Platform).Success)
            {
                filterhitPlatform = true;
            }
        }

        if (input.UseAndFilter)
        {
            if (input.UseTrainsFilter == true && input.UseDirectionsFilter == false && input.UsePlatformsFilter == false)
                return filterhitTrain;
            else if (input.UseTrainsFilter == true && input.UseDirectionsFilter == true && input.UsePlatformsFilter == false)
                return (filterhitTrain && filterhitDirection);
            else if (input.UseTrainsFilter == true && input.UseDirectionsFilter == true && input.UsePlatformsFilter == true)
                return (filterhitTrain && filterhitDirection && filterhitPlatform);
            else if (input.UseTrainsFilter == true && input.UseDirectionsFilter == false && input.UsePlatformsFilter == true)
                return (filterhitTrain && filterhitPlatform);
            else if (input.UseTrainsFilter == false && input.UseDirectionsFilter == false && input.UsePlatformsFilter == true)
                return (filterhitPlatform);
            else //if (input.useTrainsFilter == false && input.useDirectionsFilter == true && input.usePlatformsFilter == false)
                return (filterhitDirection);
        }
        else
            if (input.UseTrainsFilter == true || input.UseDirectionsFilter == true || input.UsePlatformsFilter == true)
                return (filterhitTrain || filterhitDirection || filterhitPlatform);
            else
                return true;
    }

    private string ClearLater(string input)
    {
        input = input.Replace("on time", "");
        input = input.Replace("-", "");
        input = input.Replace("n.a.", "");


        return input;
    }

    private string ClearPlatform(string input)
    {
        input = Regex.Replace(input, "<br>", "", RegexOptions.IgnoreCase | RegexOptions.Singleline);

        input = input.Replace("Platf.", "");

        return input;
    }

    private string ClearTime(string input)
    {
        input = input.Replace(" ", "");

        return input;
    }

    private string ClearDirection(string input)
    {
        input = input.Replace("<", "");
        input = input.Replace(">", "");

        return input;
    }

    private string ClearTrain(string input)
    {
        input = input.Replace("S     ", "");
        input = input.Replace("S    ", "");

        input = input.Replace("U     ", "");
        input = input.Replace("U    ", "");

        input = input.Replace(" ", "");

        return input;
    }

    #endregion Private Methods

}
