using System;
using System.Net;
using System.Text;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Drawing;

namespace DirkRettschlag.Utils
{
    public static class Utilities
    {

        public static string AppPath()
        {
            return System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
        }

        public static Size getScaledImageDimensions(int currentImageWidth, int currentImageHeight, int desiredImageWidth, int desiredImageHeight)
        {
            double scaleImageMultiplier = Math.Max((double)desiredImageWidth, (double)desiredImageHeight) / Math.Max((double)currentImageWidth, (double)currentImageHeight);

            return new Size((int)(currentImageWidth * scaleImageMultiplier), (int)(currentImageHeight * scaleImageMultiplier));
        }

        public static Bitmap ResizeBitmap(Bitmap bitmap, int currentImageWidth, int currentImageHeight, int desiredImageWidth, int desiredImageHeight)
        {
            Size fitImageSize = getScaledImageDimensions(bitmap.Width, bitmap.Height, desiredImageWidth, desiredImageHeight);
           
            Bitmap bPreviewResized = new Bitmap(fitImageSize.Width, fitImageSize.Height);

            Graphics gPreviewResized = Graphics.FromImage(bPreviewResized);

            gPreviewResized.DrawImage(bitmap, new Rectangle(0, 0, bPreviewResized.Width, bPreviewResized.Height), new Rectangle(0, 0, bitmap.Width, bitmap.Height), GraphicsUnit.Pixel);

            gPreviewResized.Dispose();

            return bPreviewResized;
        }

        public static bool Connected()
        {
            bool ret;
            try
            {
                string HostName = Dns.GetHostName();
                IPHostEntry thisHost = Dns.GetHostEntry(HostName);
                string thisIpAddr = thisHost.AddressList[0].ToString();
                ret = thisIpAddr != IPAddress.Parse("127.0.0.1").ToString();
            }
            catch
            {
                return false;
            }
            return ret;
        }

        public static string ClearText(string input)
        {
            StringBuilder sb = new StringBuilder(input);

            sb.Replace("\n", "");
            sb.Replace("\r", "");

            sb.Replace("&nbsp;", " ");		//no-breakspace=non-breakingspace
            //sb.Replace("&iexcl;", "");		//invertedexclamationmark
            //sb.Replace("&cent;", "");		//centsign
            //sb.Replace("&pound;", "");		//poundsign
            //sb.Replace("&curren;", "");		//currencysign
            //sb.Replace("&yen;", "");		//yensign=yuansign
            //sb.Replace("&brvbar;", "");		//brokenbar=brokenverticalbar
            //sb.Replace("&sect;", "");		//sectionsign
            //sb.Replace("&uml;", "");		//diaeresis=spacingdiaeresis(Umlaut)
            //sb.Replace("&copy;", "");		//copyrightsign
            //sb.Replace("&ordf;", "");		//feminineordinalindicator
            //sb.Replace("&laquo;", "");		//left-pointingdoubleanglequotationmark
            //sb.Replace("&not;", "");		//notsign
            //sb.Replace("&shy;", "");		//softhyphen=discretionaryhyphen
            //sb.Replace("&reg;", "");		//registeredsign=registeredtrademarksig
            //sb.Replace("&macr;", "");		//macron=spacingmacron=overline=APLov
            //sb.Replace("&deg;", "");		//degreesign
            //sb.Replace("&plusmn;", "");		//plus-minussign=plus-or-minussign
            //sb.Replace("&sup2;", "");		//superscripttwo=superscriptdigittwo
            //sb.Replace("&sup3;", "");		//superscriptthree=superscriptdigitthree
            //sb.Replace("&acute;", "");		//acuteaccent=spacingacute
            //sb.Replace("&micro;", "");		//microsign
            //sb.Replace("&para;", "");		//pilcrowsign=paragraphsign
            //sb.Replace("&middot;", "");		//middledot=Georgiancomma=Greekmiddle
            //sb.Replace("&cedil;", "");		//cedilla=spacingcedilla
            //sb.Replace("&sup1;", "");		//superscriptone=superscriptdigitone
            //sb.Replace("&ordm;", "");		//masculineordinalindicator
            //sb.Replace("&raquo;", "");		//right-pointingdoubleanglequotationmark
            //sb.Replace("&frac14;", "");		//vulgarfractiononequarter
            //sb.Replace("&frac12;", "");		//vulgarfractiononehalf
            //sb.Replace("&frac34;", "");		//vulgarfractionthreequarters
            //sb.Replace("&iquest;", "");		//invertedquestionmark
            //sb.Replace("&Agrave;", "");		//latincapitalletterAwithgrave
            //sb.Replace("&Aacute;", "");		//latincapitalletterAwithacute
            //sb.Replace("&Acirc;", "");		//latincapitalletterAwithcircumflex
            //sb.Replace("&Atilde;", "");		//latincapitalletterAwithtilde
            sb.Replace("&Auml;", "");		//latincapitalletterAwithdiaeresis
            //sb.Replace("&Aring;", "");		//latincapitalletterAwithringabove
            //sb.Replace("&AElig;", "");		//latincapitalletterAE
            //sb.Replace("&Ccedil;", "");		//latincapitalletterCwithcedilla
            //sb.Replace("&Egrave;", "");		//latincapitalletterEwithgrave
            //sb.Replace("&Eacute;", "");		//latincapitalletterEwithacute
            //sb.Replace("&Ecirc;", "");		//latincapitalletterEwithcircumflex
            //sb.Replace("&Euml;", "");		//latincapitalletterEwithdiaeresis
            //sb.Replace("&Igrave;", "");		//latincapitalletterIwithgrave
            //sb.Replace("&Iacute;", "");		//latincapitalletterIwithacute
            //sb.Replace("&Icirc;", "");		//latincapitalletterIwithcircumflex
            //sb.Replace("&Iuml;", "");		//latincapitalletterIwithdiaeresis
            //sb.Replace("&ETH;", "");		//latincapitalletterETH
            //sb.Replace("&Ntilde;", "");		//latincapitalletterNwithtilde
            //sb.Replace("&Ograve;", "");		//latincapitalletterOwithgrave
            //sb.Replace("&Oacute;", "");		//latincapitalletterOwithacute
            //sb.Replace("&Ocirc;", "");		//latincapitalletterOwithcircumflex
            //sb.Replace("&Otilde;", "");		//latincapitalletterOwithtilde
            sb.Replace("&Ouml;", "");		//latincapitalletterOwithdiaeresis
            //sb.Replace("&times;", "");		//multiplicationsign
            //sb.Replace("&Oslash;", "");		//latincapitalletterOwithstroke
            //sb.Replace("&Ugrave;", "");		//latincapitalletterUwithgrave
            //sb.Replace("&Uacute;", "");		//latincapitalletterUwithacute
            //sb.Replace("&Ucirc;", "");		//latincapitalletterUwithcircumflex
            sb.Replace("&Uuml;", "");		//latincapitalletterUwithdiaeresis
            //sb.Replace("&Yacute;", "");		//latincapitalletterYwithacute
            //sb.Replace("&THORN;", "");		//latincapitalletterTHORN
            sb.Replace("&szlig;", "");		//latinsmalllettersharps=ess-zed
            //sb.Replace("&agrave;", "");		//latinsmallletterawithgrave
            //sb.Replace("&aacute;", "");		//latinsmallletterawithacute
            //sb.Replace("&acirc;", "");		//latinsmallletterawithcircumflex
            //sb.Replace("&atilde;", "");		//latinsmallletterawithtilde
            sb.Replace("&auml;", "");		//latinsmallletterawithdiaeresis
            //sb.Replace("&aring;", "");		//latinsmallletterawithringabove
            //sb.Replace("&aelig;", "");		//latinsmallletterae
            //sb.Replace("&ccedil;", "");		//latinsmalllettercwithcedilla
            //sb.Replace("&egrave;", "");		//latinsmallletterewithgrave
            //sb.Replace("&eacute;", "");		//latinsmallletterewithacute
            //sb.Replace("&ecirc;", "");		//latinsmallletterewithcircumflex
            //sb.Replace("&euml;", "");		//latinsmallletterewithdiaeresis
            //sb.Replace("&igrave;", "");		//latinsmallletteriwithgrave
            //sb.Replace("&iacute;", "");		//latinsmallletteriwithacute
            //sb.Replace("&icirc;", "");		//latinsmallletteriwithcircumflex
            //sb.Replace("&iuml;", "");		//latinsmallletteriwithdiaeresis
            //sb.Replace("&eth;", "");		//latinsmalllettereth
            //sb.Replace("&ntilde;", "");		//latinsmallletternwithtilde
            //sb.Replace("&ograve;", "");		//latinsmallletterowithgrave
            //sb.Replace("&oacute;", "");		//latinsmallletterowithacute
            //sb.Replace("&ocirc;", "");		//latinsmallletterowithcircumflex
            //sb.Replace("&otilde;", "");		//latinsmallletterowithtilde
            sb.Replace("&ouml;", "");		//latinsmallletterowithdiaeresis
            //sb.Replace("&divide;", "");		//divisionsign
            //sb.Replace("&oslash;", "");		//latinsmallletterowithstroke
            //sb.Replace("&ugrave;", "");		//latinsmallletteruwithgrave
            //sb.Replace("&uacute;", "");		//latinsmallletteruwithacute
            //sb.Replace("&ucirc;", "");		//latinsmallletteruwithcircumflex
            sb.Replace("&uuml;", "");		//latinsmallletteruwithdiaeresis
            //sb.Replace("&yacute;", "");		//latinsmallletterywithacute
            //sb.Replace("&thorn;", "");		//latinsmallletterthorn
            //sb.Replace("&yuml;", "");		//latinsmallletterywithdiaeresis
            sb.Replace("&#160;", " ");		//no-breakspace=non-breakingspace
            //sb.Replace("&#161;", "");		//invertedexclamationmark
            //sb.Replace("&#162;", "");		//centsign
            //sb.Replace("&#163;", "");		//poundsign
            //sb.Replace("&#164;", "");		//currencysign
            //sb.Replace("&#165;", "");		//yensign=yuansign
            //sb.Replace("&#166;", "");		//brokenbar=brokenverticalbar
            //sb.Replace("&#167;", "");		//sectionsign
            //sb.Replace("&#168;", "");		//diaeresis=spacingdiaeresis(Umlaut)
            //sb.Replace("&#169;", "");		//copyrightsign
            //sb.Replace("&#170;", "");		//feminineordinalindicator
            //sb.Replace("&#171;", "");		//left-pointingdoubleanglequotationmark
            //sb.Replace("&#172;", "");		//notsign
            //sb.Replace("&#173;", "");		//softhyphen=discretionaryhyphen
            //sb.Replace("&#174;", "");		//registeredsign=registeredtrademarksig
            //sb.Replace("&#175;", "");		//macron=spacingmacron=overline=APLov
            //sb.Replace("&#176;", "");		//degreesign
            //sb.Replace("&#177;", "");		//plus-minussign=plus-or-minussign
            //sb.Replace("&#178;", "");		//superscripttwo=superscriptdigittwo
            //sb.Replace("&#179;", "");		//superscriptthree=superscriptdigitthree
            //sb.Replace("&#180;", "");		//acuteaccent=spacingacute
            //sb.Replace("&#181;", "");		//microsign
            //sb.Replace("&#182;", "");		//pilcrowsign=paragraphsign
            //sb.Replace("&#183;", "");		//middledot=Georgiancomma=Greekmiddle
            //sb.Replace("&#184;", "");		//cedilla=spacingcedilla
            //sb.Replace("&#185;", "");		//superscriptone=superscriptdigitone
            //sb.Replace("&#186;", "");		//masculineordinalindicator
            //sb.Replace("&#187;", "");		//right-pointingdoubleanglequotationmark
            //sb.Replace("&#188;", "");		//vulgarfractiononequarter
            //sb.Replace("&#189;", "");		//vulgarfractiononehalf
            //sb.Replace("&#190;", "");		//vulgarfractionthreequarters
            //sb.Replace("&#191;", "");		//invertedquestionmark
            //sb.Replace("&#192;", "");		//latincapitalletterAwithgrave
            //sb.Replace("&#193;", "");		//latincapitalletterAwithacute
            //sb.Replace("&#194;", "");		//latincapitalletterAwithcircumflex
            //sb.Replace("&#195;", "");		//latincapitalletterAwithtilde
            sb.Replace("&#196;", "");		//latincapitalletterAwithdiaeresis
            //sb.Replace("&#197;", "");		//latincapitalletterAwithringabove
            //sb.Replace("&#198;", "");		//latincapitalletterAE
            //sb.Replace("&#199;", "");		//latincapitalletterCwithcedilla
            //sb.Replace("&#200;", "");		//latincapitalletterEwithgrave
            //sb.Replace("&#201;", "");		//latincapitalletterEwithacute
            //sb.Replace("&#202;", "");		//latincapitalletterEwithcircumflex
            //sb.Replace("&#203;", "");		//latincapitalletterEwithdiaeresis
            //sb.Replace("&#204;", "");		//latincapitalletterIwithgrave
            //sb.Replace("&#205;", "");		//latincapitalletterIwithacute
            //sb.Replace("&#206;", "");		//latincapitalletterIwithcircumflex
            //sb.Replace("&#207;", "");		//latincapitalletterIwithdiaeresis
            //sb.Replace("&#208;", "");		//latincapitalletterETH
            //sb.Replace("&#209;", "");		//latincapitalletterNwithtilde
            //sb.Replace("&#210;", "");		//latincapitalletterOwithgrave
            //sb.Replace("&#211;", "");		//latincapitalletterOwithacute
            //sb.Replace("&#212;", "");		//latincapitalletterOwithcircumflex
            //sb.Replace("&#213;", "");		//latincapitalletterOwithtilde
            sb.Replace("&#214;", "");		//latincapitalletterOwithdiaeresis
            //sb.Replace("&#215;", "");		//multiplicationsign
            //sb.Replace("&#216;", "");		//latincapitalletterOwithstroke
            //sb.Replace("&#217;", "");		//latincapitalletterUwithgrave
            //sb.Replace("&#218;", "");		//latincapitalletterUwithacute
            //sb.Replace("&#219;", "");		//latincapitalletterUwithcircumflex
            sb.Replace("&#220;", "");		//latincapitalletterUwithdiaeresis
            //sb.Replace("&#221;", "");		//latincapitalletterYwithacute
            //sb.Replace("&#222;", "");		//latincapitalletterTHORN
            sb.Replace("&#223;", "");		//latinsmalllettersharps=ess-zed
            //sb.Replace("&#224;", "");		//latinsmallletterawithgrave
            //sb.Replace("&#225;", "");		//latinsmallletterawithacute
            //sb.Replace("&#226;", "");		//latinsmallletterawithcircumflex
            //sb.Replace("&#227;", "");		//latinsmallletterawithtilde
            sb.Replace("&#228;", "");		//latinsmallletterawithdiaeresis
            //sb.Replace("&#229;", "");		//latinsmallletterawithringabove
            //sb.Replace("&#230;", "");		//latinsmallletterae
            //sb.Replace("&#231;", "");		//latinsmalllettercwithcedilla
            //sb.Replace("&#232;", "");		//latinsmallletterewithgrave
            //sb.Replace("&#233;", "");		//latinsmallletterewithacute
            //sb.Replace("&#234;", "");		//latinsmallletterewithcircumflex
            //sb.Replace("&#235;", "");		//latinsmallletterewithdiaeresis
            //sb.Replace("&#236;", "");		//latinsmallletteriwithgrave
            //sb.Replace("&#237;", "");		//latinsmallletteriwithacute
            //sb.Replace("&#238;", "");		//latinsmallletteriwithcircumflex
            //sb.Replace("&#239;", "");		//latinsmallletteriwithdiaeresis
            //sb.Replace("&#240;", "");		//latinsmalllettereth
            //sb.Replace("&#241;", "");		//latinsmallletternwithtilde
            //sb.Replace("&#242;", "");		//latinsmallletterowithgrave
            //sb.Replace("&#243;", "");		//latinsmallletterowithacute
            //sb.Replace("&#244;", "");		//latinsmallletterowithcircumflex
            //sb.Replace("&#245;", "");		//latinsmallletterowithtilde
            sb.Replace("&#246;", "");		//latinsmallletterowithdiaeresis
            //sb.Replace("&#247;", "");		//divisionsign
            //sb.Replace("&#248;", "");		//latinsmallletterowithstroke
            //sb.Replace("&#249;", "");		//latinsmallletteruwithgrave
            //sb.Replace("&#250;", "");		//latinsmallletteruwithacute
            //sb.Replace("&#251;", "");		//latinsmallletteruwithcircumflex
            sb.Replace("&#252;", "");		//latinsmallletteruwithdiaeresis
            //sb.Replace("&#253;", "");		//latinsmallletterywithacute
            //sb.Replace("&#254;", "");		//latinsmallletterthorn
            //sb.Replace("&#255;", "");		//latinsmallletterywithdiaeresis

            return sb.ToString();
        }

        static public string Replace(string original, string pattern, string replacement, StringComparison comparisonType)
        {
            if (original == null)
            {
                return null;
            }

            if (String.IsNullOrEmpty(pattern))
            {
                return original;
            }

            int lenPattern = pattern.Length;
            int idxPattern = -1;
            int idxLast = 0;

            StringBuilder result = new StringBuilder();

            while (true)
            {
                idxPattern = original.IndexOf(pattern, idxPattern + 1, comparisonType);

                if (idxPattern < 0)
                {
                    result.Append(original, idxLast, original.Length - idxLast);

                    break;
                }

                result.Append(original, idxLast, idxPattern - idxLast);
                result.Append(replacement);

                idxLast = idxPattern + lenPattern;
            }

            return result.ToString();
        }

        [DllImport("coredll.dll", SetLastError = true)]
        public static extern IntPtr FindWindow(string ClassName, string WindowName);

        private static Int32 FILE_DEVICE_HAL = 0x00000101;
        private static Int32 FILE_ANY_ACCESS = 0x0;
        private static Int32 METHOD_BUFFERED = 0x0;

        private static Int32 IOCTL_HAL_GET_DEVICEID =
            ((FILE_DEVICE_HAL) << 16) | ((FILE_ANY_ACCESS) << 14)
             | ((21) << 2) | (METHOD_BUFFERED);

        [DllImport("coredll.dll")]
        private static extern bool KernelIoControl(Int32 IoControlCode, IntPtr
          InputBuffer, Int32 InputBufferSize, byte[] OutputBuffer, Int32
          OutputBufferSize, ref Int32 BytesReturned);

        public static string GetDeviceID()
        {
            byte[] OutputBuffer = new byte[256];
            Int32 OutputBufferSize, BytesReturned;
            OutputBufferSize = OutputBuffer.Length;
            BytesReturned = 0;

            // Call KernelIoControl passing the previously defined
            // IOCTL_HAL_GET_DEVICEID parameter
            // We dont need to pass any input buffers to this call
            // so InputBuffer and InputBufferSize are set to their null
            // values
            bool retVal = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 
                    IntPtr.Zero,
                    0,
                    OutputBuffer,
                    OutputBufferSize,
                    ref BytesReturned);

            // If the request failed, exit the method now
            if (retVal == false)
            {
                return null;
            }
            
            // Examine the OutputBuffer byte array to find the start of the 
            // Preset ID and Platform ID, as well as the size of the
            // PlatformID. 
            // PresetIDOffset  The number of bytes the preset ID is offset
            //                  from the beginning of the structure
            // PlatformIDOffset - The number of bytes the platform ID is
            //                    offset from the beginning of the structure
            // PlatformIDSize - The number of bytes used to store the
            //                  platform ID
            // Use BitConverter.ToInt32() to convert from byte[] to int
            Int32 PresetIDOffset = BitConverter.ToInt32(OutputBuffer, 4); 
            Int32 PlatformIDOffset = BitConverter.ToInt32(OutputBuffer, 0xc);
            Int32 PlatformIDSize = BitConverter.ToInt32(OutputBuffer, 0x10);

            // Convert the Preset ID segments into a string so they can be 
            // displayed easily.
            StringBuilder sb = new StringBuilder();
            sb.Append(String.Format("{0:X8}-{1:X4}-{2:X4}-{3:X4}-", 
                 BitConverter.ToInt32(OutputBuffer, PresetIDOffset), 
                 BitConverter.ToInt16(OutputBuffer, PresetIDOffset +4), 
                 BitConverter.ToInt16(OutputBuffer, PresetIDOffset +6), 
                 BitConverter.ToInt16(OutputBuffer, PresetIDOffset +8))); 

            // Break the Platform ID down into 2-digit hexadecimal numbers
            // and append them to the Preset ID. This will result in a 
            // string-formatted Device ID
            for (int i = PlatformIDOffset; 
                 i < PlatformIDOffset +   PlatformIDSize; 
                 i ++ )  
            {
                sb.Append(String.Format("{0:X2}", OutputBuffer[i]));
            }
            
            // return the Device ID string
            return sb.ToString();
        }
    }
}
