using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Net;

using DirkRettschlag.Utils;
using System.Text.RegularExpressions;
using System.Collections;

namespace Departure
{
    public partial class frmStationSettings : Form
    {

        private HttpWebRequest request;
        private StationSettings stationSettings;
        private GeneralSettings generalSettings;
        
        string transfers;
        string transfersSelected = "000000000";

        SortedList<string, string> stations;
        string stationID = "";

        Departure.Webservice.WebserviceWrapper ws = new Departure.Webservice.WebserviceWrapper();
        
        public StationSettings StationSettings
        {
            get
            {
                return this.stationSettings;
            }
            set
            {
                this.stationSettings = value;
                this.stationSettings.getSettings();
                this.setControls();
            }
        }

        public GeneralSettings GeneralSettings
        {
            set { this.generalSettings = value; }
        }

        public frmStationSettings()
        {
            InitializeComponent();

            if (Platform.GetDeviceType() != "Smartphone")
            {
                InitializeComponentPPC();
            }

            buttonStation_Click(this, EventArgs.Empty);
        }

        private void setControls()
        {
            txtStation.Text = this.stationSettings.Station;
            txtAbbreviation.Text = this.stationSettings.Abbreviation;

            string tmpTransfers = this.stationSettings.TransfersSelected;

            if (tmpTransfers[0].ToString() == "1") { chkICE.Checked = true; } else { chkICE.Checked = false; }
            if (tmpTransfers[1].ToString() == "1") { chkECIC.Checked = true; } else { chkECIC.Checked = false; }
            if (tmpTransfers[2].ToString() == "1") { chkIR.Checked = true; } else { chkIR.Checked = false; }
            if (tmpTransfers[3].ToString() == "1") { chkRERB.Checked = true; } else { chkRERB.Checked = false; }
            if (tmpTransfers[4].ToString() == "1") { chkSBahn.Checked = true; } else { chkSBahn.Checked = false; }
            if (tmpTransfers[5].ToString() == "1") { chkBus.Checked = true; } else { chkBus.Checked = false; }
            if (tmpTransfers[6].ToString() == "1") { chkBoat.Checked = true; } else { chkBoat.Checked = false; }
            if (tmpTransfers[7].ToString() == "1") { chkUBahn.Checked = true; } else { chkUBahn.Checked = false; }
            if (tmpTransfers[8].ToString() == "1") { chkTram.Checked = true; } else { chkTram.Checked = false; }

            setCheckAndBGColor(chkICE, panelICE, true);
            setCheckAndBGColor(chkECIC, panelECIC, true);
            setCheckAndBGColor(chkIR, panelIR, true);
            setCheckAndBGColor(chkRERB, panelRERB, true);
            setCheckAndBGColor(chkSBahn, panelSBahn, true);
            setCheckAndBGColor(chkBus, panelBus, true);
            setCheckAndBGColor(chkBoat, panelBoat, true);
            setCheckAndBGColor(chkUBahn, panelUBahn, true);
            setCheckAndBGColor(chkTram, panelTram, true);

            chkTrainsFilter.Checked = this.stationSettings.UseTrainsFilter;
            txtTrainsFilter.Text = this.stationSettings.TrainsFilter;

            chkDirectionsFilter.Checked = this.stationSettings.UseDirectionsFilter;
            txtDirectionsFilter.Text = this.stationSettings.DirectionsFilter;

            chkPlatformsFilter.Checked = this.stationSettings.UsePlatformsFilter;
            txtPlatformsFilter.Text = this.stationSettings.PlatformsFilter;

            rbtFilterLogicAND.Checked = this.stationSettings.UseAndFilter;
            rbtFilterLogicOR.Checked = !rbtFilterLogicAND.Checked;

            chkShowPlatform.Checked = this.stationSettings.UseShowPlatform;
        }

        private void setStationSettings()
        {
            this.stationSettings.Station = txtStation.Text;
            this.stationSettings.StationID = this.stationID;
            this.stationSettings.Abbreviation = txtAbbreviation.Text;
            this.stationSettings.Transfers = transfers;
            this.stationSettings.TransfersSelected = transfersSelected;
            this.stationSettings.UseTrainsFilter = chkTrainsFilter.Checked;
            this.stationSettings.TrainsFilter = txtTrainsFilter.Text;
            this.stationSettings.UseDirectionsFilter = chkDirectionsFilter.Checked;
            this.stationSettings.DirectionsFilter = txtDirectionsFilter.Text;
            this.stationSettings.UsePlatformsFilter = chkPlatformsFilter.Checked;
            this.stationSettings.PlatformsFilter = txtPlatformsFilter.Text;
            this.stationSettings.UseAndFilter = rbtFilterLogicAND.Checked;
            this.stationSettings.UseShowPlatform = chkShowPlatform.Checked;
        }

        private void mniSave_Click(object sender, EventArgs e)
        {
            this.setStationSettings();
            this.stationSettings.saveSettings();

            this.cboChoose.Visible = false;
            this.lblChoose.Text = "";

            this.Close();
        }

        private void mniCancel_Click(object sender, EventArgs e)
        {
            this.stationSettings.getSettings();
            this.setControls();

            this.cboChoose.Visible = false;
            this.lblChoose.Text = "";

            this.Close();
        }

        private void inputPanel_EnabledChanged(object sender, EventArgs e)
        {
            if (this.inputPanel.Enabled)
            {
                this.panelSettings.Height -= this.inputPanel.Bounds.Height;
            }
            else
            {
                this.panelSettings.Height = this.ClientSize.Height;
            }
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            try
            {

                if (Utilities.Connected() || generalSettings.AutoConnect)
                {

                    Cursor.Current = Cursors.WaitCursor;


                    this.lblChoose.Text = "Searching...";
                    this.lblChoose.Refresh();
                    this.cboChoose.Items.Clear();

                    Application.DoEvents();

                    if (generalSettings.UseWebService)
                    {
                        ws.setURL(generalSettings.WebServiceURL);
                        ws.BeginFindStation(generalSettings.DeviceID, this.txtStation.Text, new AsyncCallback(FindStationCallback), ws);
                    }
                    else
                    {
                        request = (HttpWebRequest)WebRequest.Create("http://mobile.bahn.de/bin/mobil/bhftafel.exe/en?input=" + this.txtStation.Text + "&time=actual&boardType=dep&productsFilter=0000001000000000&start=Suchen&boardType=Abfahrt&ao=yes");

                        request.Credentials = CredentialCache.DefaultCredentials;
                        request.KeepAlive = true;
                        request.Method = "GET";

                        if (generalSettings.UserAgent != "")
                            request.UserAgent = generalSettings.UserAgent;

                        if (generalSettings.HTTPHeader != "")
                            request.Headers.Add(generalSettings.HTTPHeader);

                        HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                        System.IO.StreamReader sr = new System.IO.StreamReader(response.GetResponseStream());

                        string source = sr.ReadToEnd();

                        sr.Close();

                        if (source.IndexOf("Your input is ambiguous.") >= 0)
                        {
                            Regex regex = new Regex("<select class=\"error\" id=\"rplc0\".*?</select>", RegexOptions.Singleline | RegexOptions.IgnoreCase);
                            Match match;
                            MatchCollection matchcollection;

                            stations = new SortedList<string, string>();

                            match = regex.Match(source);

                            if (match.Success)
                            {

                                regex = new Regex("<option.*?>.*?</option>", RegexOptions.IgnoreCase);

                                matchcollection = regex.Matches(match.Value);
                                string station;
                                
                                foreach (Match tmpmatch in matchcollection)
                                {
                                    station = Regex.Replace(Utilities.ClearText(tmpmatch.Value), "<option value=\"", "");
                                    station = Regex.Replace(station, "\".*", "");
                                    stations.Add(station.Split('#')[1], station.Split('#')[0]);
                                    this.cboChoose.Items.Add(station.Split('#')[0]);
                                }

                                this.lblChoose.Text = "Multiple Stations found:";

                                this.cboChoose.SelectedIndex = 0;
                                this.cboChoose.Visible = true;

                            }
                            else
                            {
                                this.lblChoose.Text = "Error.";
                            }

                        }
                        else
                        {
                            Regex regex = new Regex("<h1 class=\"pagetitle\"><a.*?input=(?<Name>.*?)\\&.*?</h1>", RegexOptions.Singleline | RegexOptions.IgnoreCase);
                            Match match;
                            match = regex.Match(source);
                            if (match.Success)
                            {
                                this.txtStation.Text = Utilities.ClearText(match.Groups["Name"].Value.Replace("%23", "#").Split('#')[0]);
                                this.stationID = match.Groups["Name"].Value.Replace("%23", "#").Split('#')[1];
                            }

                            this.lblChoose.Text = "Station found.";
                        }

                        Cursor.Current = Cursors.Default;
                    }
                }

            }
            catch (Exception ex)
            {
                Cursor.Current = Cursors.Default;

                MessageBox.Show(ex.Message, "An error occured");

            }

        }

        private void FindStationCallback(IAsyncResult asyncResult)
        {
            try
            {

                ws = (Webservice.WebserviceWrapper)asyncResult.AsyncState;

                List<string> stations = ws.EndFindStation(asyncResult);

                this.stations = new SortedList<string, string>();

                if (stations.Count > 1)
                {
                    string tmpStation;

                    foreach (string station in stations)
                    {
                        tmpStation = Utilities.ClearText(station);
                        this.stations.Add(tmpStation.Split('#')[1], tmpStation.Split('#')[0]);
                        Invoking.AddComboBoxItem(this.cboChoose, tmpStation.Split('#')[0]);
                    }

                    Invoking.SetText(this.lblChoose, "Multiple Stations found:");

                    Invoking.SetSelectedIndex(this.cboChoose, 0);
                    Invoking.SetControlVisible(this.cboChoose, true);
                }
                else if (stations.Count == 1)
                {
                    Invoking.SetText(this.txtStation, Utilities.ClearText(stations[0].Split('#')[0]));
                    this.stationID = stations[0].Split('#')[1];
                    Invoking.SetText(this.lblChoose, "Station found.");
                }
                else
                {
                    Invoking.SetText(this.lblChoose, "Error.");
                }

            }
            catch (Exception ex)
            {
                Invoking.SetCursor(this, Cursors.Default);

                MessageBox.Show(ex.Message, "An error occured");

            }

            Invoking.SetCursor(this, Cursors.Default);

        }

        private void cboChoose_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.txtStation.Text = this.cboChoose.SelectedItem.ToString();
            this.stationID = stations.Keys[stations.IndexOfValue(this.cboChoose.SelectedItem.ToString())];
        }

        private void TransportsCheckStateChanged()
        {
            transfers = "";

            if (chkICE.Checked) transfers += "&GUIREQProduct_0=on";
            if (chkECIC.Checked) transfers += "&GUIREQProduct_1=on";
            if (chkIR.Checked) transfers += "&GUIREQProduct_2=on";
            if (chkRERB.Checked) transfers += "&GUIREQProduct_3=on";
            if (chkSBahn.Checked) transfers += "&GUIREQProduct_4=on";
            if (chkBus.Checked) transfers += "&GUIREQProduct_5=on";
            if (chkBoat.Checked) transfers += "&GUIREQProduct_6=on";
            if (chkUBahn.Checked) transfers += "&GUIREQProduct_7=on";
            if (chkTram.Checked) transfers += "&GUIREQProduct_8=on";

            transfersSelected = "";

            if (chkICE.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkECIC.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkIR.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkRERB.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkSBahn.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkBus.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkBoat.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkUBahn.Checked) transfersSelected += 1; else transfersSelected += 0;
            if (chkTram.Checked) transfersSelected += 1; else transfersSelected += 0;

        }

        private void setCheckAndBGColor(CheckBox checkbox, Panel panel, bool init)
        {
            if (!init)
                checkbox.Checked = !checkbox.Checked;

            if (checkbox.Checked)
                panel.BackColor = Color.Green;
            else
                panel.BackColor = Color.Red;

            TransportsCheckStateChanged();
        }

        private void panelICE_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkICE, panelICE, false);
        }

        private void panelECIC_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkECIC, panelECIC, false);
        }

        private void panelIR_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkIR, panelIR, false);
        }

        private void panelRERB_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkRERB, panelRERB, false);
        }

        private void panelSBahn_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkSBahn, panelSBahn, false);
        }

        private void panelUBahn_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkUBahn, panelUBahn, false);
        }

        private void panelTram_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkTram, panelTram, false);
        }

        private void panelBus_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkBus, panelBus, false);
        }

        private void panelBoat_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkBoat, panelBoat, false);
        }

        private void panelSettingsTransports_Resize(object sender, EventArgs e)
        {
            if (panelSettingsTransports.Height > panelTransports.Height)
                panelTransports.Top = (panelSettingsTransports.Height / 2) - (panelTransports.Height / 2);
            else
                panelTransports.Top = 0;

            if (panelSettingsTransports.Width > panelTransports.Width)
                panelTransports.Left = (panelSettingsTransports.Width / 2) - (panelTransports.Width / 2);
            else
                panelTransports.Left = 0;
        }

        private void buttonStation_Click(object sender, EventArgs e)
        {
            panelSettingsStation.Visible = true;
            panelSettingsTransports.Visible = false;
            panelSettingsFilter.Visible = false;
        }

        private void buttonTransports_Click(object sender, EventArgs e)
        {
            panelSettingsTransports.Visible = true;
            panelSettingsFilter.Visible = false;
            panelSettingsStation.Visible = false;
        }

        private void buttonFilter_Click(object sender, EventArgs e)
        {
            panelSettingsFilter.Visible = true;
            panelSettingsTransports.Visible = false;
            panelSettingsStation.Visible = false;
        }

        private void chkICE_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkICE, panelICE, true);
        }

        private void chkECIC_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkECIC, panelECIC, true);
        }

        private void chkIR_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkIR, panelIR, true);
        }

        private void chkRERB_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkRERB, panelRERB, true);
        }

        private void chkSBahn_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkSBahn, panelSBahn, true);
        }

        private void chkUBahn_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkUBahn, panelUBahn, true);
        }

        private void chkTram_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkTram, panelTram, true);
        }

        private void chkBus_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkBus, panelBus, true);
        }

        private void chkBoat_Click(object sender, EventArgs e)
        {
            setCheckAndBGColor(chkBoat, panelBoat, true);
        }

        private void chk_GotFocus(object sender, EventArgs e)
        {
            if ((sender as CheckBox).Checked)
                (sender as CheckBox).Parent.BackColor = Color.FromArgb(0, 128, 0);
            else
                (sender as CheckBox).Parent.BackColor = Color.FromArgb(128, 0, 0);
        }

        private void chk_LostFocus(object sender, EventArgs e)
        {
            if ((sender as CheckBox).Checked)
                (sender as CheckBox).Parent.BackColor = Color.Green;
            else
                (sender as CheckBox).Parent.BackColor = Color.Red;
        }

    }
}
